"""

.. currentmodule:: basin3d.plugins.usgs

:platform: Unix, Mac
:synopsis: USGS Daily Values and Instantaneous Values Plugin Definition and supporting views.
:module author: Val Hendrix <vhendrix@lbl.gov>
:module author: Danielle S Christianson <dschristianson@lbl.gov>


* :class:`USGSDataSourcePlugin` - This Data Source plugin maps the USGS Daily Values and Instantaneous Values Service to BASIN-3D Models

USGS to BASIN-3D Mapping
++++++++++++++++++++++++
The table below describes how BASIN-3D synthesis models are mapped to the USGS Daily Values and Instantaneous Service models.

=================== === ==================================================================================================
USGS NWIS               BASIN-3D
=================== === ==================================================================================================
``nwis/dv``         >>  :class:`basin3d.core.models.MeasurementTimeseriesTVPObservation` with aggregation_duration == DAY
------------------- --- --------------------------------------------------------------------------------------------------
``nwis/iv``         >>  :class:`basin3d.core.models.MeasurementTimeseriesTVPObservation` with aggregation_duration == NONE
------------------- --- --------------------------------------------------------------------------------------------------
``nwis/sites``      >>  :class:`basin3d.synthesis.models.field.MonitoringFeature`
------------------- --- --------------------------------------------------------------------------------------------------
``nwis/huc``        >>  :class:`basin3d.core.models.MonitoringFeature`
------------------- --- --------------------------------------------------------------------------------------------------
``new_huc_rdb.txt`` >>  :class:`basin3d.synthesis.models.field.MonitoringFeature`
                         * Region (2-digit HUC code) to Region
                         * Subregion (4-digit HUC code) to Subregion
                         * Accounting (6-digit HUC code) Unit to Basin
                         * Watershed (8-digit HUC code) to Subbasin
=================== === ==================================================================================================


Access Classes
++++++++++++++

The following are the access classes that map *USGS Water Data* to the *BASIN-3D Models*.

* :class:`USGSMeasurementTimeseriesTVPObservationAccess` - Access for accessing a group of data points grouped by time, space, model, sample  etc.
* :class:`USGSMonitoringFeatureAccess` - Access for accessing monitoring features



---------------------
"""
import json

from basin3d.core import monitor
from typing import Any, List, Optional, Tuple

import requests

from basin3d.core.schema.enum import FeatureTypeEnum, AggregationDurationEnum
from basin3d.core.schema.query import QueryMeasurementTimeseriesTVP, QueryMonitoringFeature
from basin3d.core.access import get_url
from basin3d.core.models import AbsoluteCoordinate, AltitudeCoordinate, Coordinate, GeographicCoordinate, \
    MeasurementTimeseriesTVPObservation, MonitoringFeature, RelatedSamplingFeature, \
    TimeMetadataMixin, TimeValuePair, ResultListTVP
from basin3d.core.plugin import DataSourcePluginAccess, DataSourcePluginPoint, basin3d_plugin
from basin3d.core.types import SpatialSamplingShapes
from basin3d.plugins import usgs_huc_codes

logger = monitor.get_logger(__name__)

URL_USGS_HUC = "https://water.usgs.gov/GIS/new_huc_rdb.txt"


def convert_discharge(data, data_str, parameter, units):
    """
    Convert the River Discharge to m^3
    :param data:
    :param data_str:
    :param parameter:
    :param units:
    :return:
    """
    if parameter in ['00060', '00061']:
        # Hardcode conversion from ft^3 to m^3
        # for River discharge
        if data_str == '-999999':
            data = int(data_str)
        else:
            data *= 0.028316847
        units = "m^3/s"
    return data, units


def generator_usgs_measurement_timeseries_tvp_observation(view,
                                                          query: QueryMeasurementTimeseriesTVP,
                                                          synthesis_messages):
    """
    Get the data for USGS Daily Values or Instantaneous Values

    :param view: Access class
    :param query: Query information for this request
    :returns: a generator object that yields data from the request in json form
    """

    # Temporal resolution is always daily.
    search_params: List[Tuple[str, Any]] = list()

    search_params.append(("startDT", query.start_date))

    if query.end_date:
        search_params.append(("endDT", query.end_date))

    search_params.append(("parameterCd", ",".join([str(o) for o in query.observed_property])))

    if query.statistic:
        # if aggregation duration is NONE (iv) and there is a query that has a stat param, clear the statistics list
        # add warning message to user, but there are no statistic values for IV call. need synthesis param to add message
        if query.aggregation_duration[0] == AggregationDurationEnum.NONE:
            synthesis_messages.append(
                f"USGS Instantaneous Values service does not support statistics and cannot be specified when aggregation_duration = {AggregationDurationEnum.NONE}. Specified statistic arguments will be ignored.")
            logger.info(
                f"USGS Instantaneous Values service does not support statistics and cannot be specified when aggregation_duration = {AggregationDurationEnum.NONE}. Specified statistic arguments will be ignored.")
        else:
            search_params.append(("statCd", ",".join([str(o) for o in query.statistic])))

    else:
        search_params.append(("siteStatus", "all"))

    if len(query.monitoring_feature[0]) > 2:
        # search for stations
        search_params.append(("sites", ",".join(query.monitoring_feature)))
    else:
        # search for stations by specifying the huc
        search_params.append(("huc", ",".join(query.monitoring_feature)))

    # look for station locations only
    search_params.append(("siteType", "ST"))

    # JSON format
    search_params.append(("format", "json"))

    # Request the data points, calls IV or DV depending on aggregation duration passed in param
    # Default to DV service if aggregation duration is DAY or when nothing is specified
    # Calls IV service in aggregation duration is NONE
    endpoint = query.aggregation_duration[0] == AggregationDurationEnum.NONE and "iv" or "dv"
    response = get_url(f'{{}}{endpoint}'.format(view.datasource.location), params=search_params)

    if response.status_code == 200:
        try:

            json_obj = response.json()

            # There is a valid json response
            if json_obj:
                timeseries_json = json_obj['value']['timeSeries']

                # Iterate over monitoring_features
                for data_json in timeseries_json:
                    yield data_json

        except json.decoder.JSONDecodeError:
            synthesis_messages.append("JSON Not Returned: {}".format(response.content))
            logger.error("JSON Not Returned: {}".format(response.content))
    else:
        import re
        p = re.compile(r'<.*?>')
        synthesis_messages.append("HTTP {}: {}".format(response.status_code, p.sub(' ', response.text)))
        logger.error("HTTP {}: {}".format(response.status_code, p.sub(' ', response.text)))


def iter_rdb_to_json(rdb_text):
    """
    Generator that iterates over an rdb file

    :param rdb_text: USGS rdb data
    :return:
    """
    header = None
    format = None  # not doing anything with this
    for i, line in enumerate(rdb_text.split("\n")):
        line = line.replace("\r", "")
        line = line.lstrip()
        if line and not line[0] == '#':
            if not header:
                header = line.split()
                continue
            elif not format:
                format = line.split()

            elif line and line[0] not in ['#', ' ', '<', ' ']:
                data = line.split("\t")

                json_object = dict(zip(header, data))
                yield json_object


def _load_point_obj(datasource, json_obj, observed_property_variables, synthesis_messages):
    """
    Instantiate the object

    #
    # The following selected fields are included in this output:
    #
    #  agency_cd       -- Agency
    #  site_no         -- Site identification number
    #  station_nm      -- Site name
    #  site_tp_cd      -- Site type
    #  dec_lat_va      -- Decimal latitude
    #  dec_long_va     -- Decimal longitude
    #  coord_acy_cd    -- Latitude-longitude accuracy
    #  dec_coord_datum_cd -- Decimal Latitude-longitude datum
    #  alt_va          -- Altitude of Gage/land surface
    #  alt_acy_va      -- Altitude accuracy
    #  alt_datum_cd    -- Altitude datum
    #  huc_cd          -- Hydrologic unit code

    :param json_obj:
    :param observed_property_variables: list of locations and their available variables
    :param synthesis_messages
    :return:
    """

    if 'site_no' in json_obj:
        id = json_obj['site_no']

        lat, lon = None, None
        try:
            lat, lon = float(json_obj['dec_lat_va']), float(json_obj['dec_long_va'])
        except Exception as e:
            synthesis_messages.append(f"Error getting latlon: {str(e)}")
            logger.error(str(e))
        if id not in observed_property_variables.keys():
            mf_opv = "Could not find observed property variables for this monitoring feature"
            msg = f"Could not find observed property variables for this monitoring feature: {id}"
            synthesis_messages.append(msg)
            logger.warning({msg})
        else:
            mf_opv = observed_property_variables[id]
        monitoring_feature = MonitoringFeature(
            datasource,
            id="{}".format(id),
            name=json_obj['station_nm'],
            feature_type=FeatureTypeEnum.POINT,
            shape=SpatialSamplingShapes.SHAPE_POINT,
            related_sampling_feature_complex=[RelatedSamplingFeature(
                datasource,
                related_sampling_feature=json_obj['huc_cd'],
                related_sampling_feature_type=FeatureTypeEnum.SUBBASIN,  # previously site
                role=RelatedSamplingFeature.ROLE_PARENT
            )],
            # geographical_group_id=huc_accounting_unit_id,
            # geographical_group_type=FeatureTypeEnum.REGION,
            observed_properties=mf_opv,
            coordinates=Coordinate(
                absolute=AbsoluteCoordinate(
                    horizontal_position=GeographicCoordinate(
                        **{"latitude": lat,
                           "longitude": lon,
                           "datum": json_obj['dec_coord_datum_cd'],
                           "units": GeographicCoordinate.UNITS_DEC_DEGREES}))
            )
        )

        # huc_accounting_unit_id = json_obj['huc_cd'][0:6]  #Subbasin
        if json_obj['alt_va'] and json_obj['alt_acy_va'] and json_obj['alt_datum_cd']:
            monitoring_feature.coordinates.absolute.vertical_extent = \
                [AltitudeCoordinate(
                    **{"value": float(json_obj['alt_va']),
                       "resolution": float(json_obj['alt_acy_va']),
                       "datum": json_obj['alt_datum_cd']})]

        return monitoring_feature


def _parse_sites_response(usgs_site_response):
    """
    Get a dictionary of location variables for the given location results and
    get a dictionary of unique sites or subbasins
    :param usgs_site_response: datasource JSON object of the locations
    :return observed_properties_variables, unique_usgs_sites: a tuple of a dictionary of observed property variables for the given location results and a dictionary of unique sites or subbasins
    """
    observed_properties_variables = {}
    unique_usgs_sites = {}

    for v in iter_rdb_to_json(usgs_site_response.text):
        param, site, stat = v['parm_cd'], v['site_no'], v['stat_cd']
        observed_properties_variables.setdefault(site, [])

        # FIX: stat -- need to change to B3D vocab
        if param not in observed_properties_variables[site] and stat != 'NOT_SUPPORTED':
            observed_properties_variables[site].append(param)
        if site not in unique_usgs_sites:
            unique_usgs_sites[site] = v
    logger.debug("Location DataTypes: {}".format(observed_properties_variables))

    return observed_properties_variables, unique_usgs_sites


class USGSMonitoringFeatureAccess(DataSourcePluginAccess):
    """
    Access for mapping USGS HUC Units to :class:`~basin3d.core.models.MonitoringFeature` objects.

    ============== === =======================================================
    USGS HUC code      BASIN-3D
    ============== === =======================================================
    2-digit        >>  :class:`basin3d.core.schema.enum.FeatureType` REGION
    -------------- --- -------------------------------------------------------
    4-digit        >>  :class:`basin3d.core.schema.enum.FeatureType` SUBREGION
    -------------- --- -------------------------------------------------------
    6-digit        >>  :class:`basin3d.core.schema.enum.FeatureType` BASIN
    -------------- --- -------------------------------------------------------
    8-digit        >>  :class:`basin3d.core.schema.enum.FeatureType` SUBBASIN
    ============== === =======================================================

    """
    synthesis_model_class = MonitoringFeature

    def list(self, query: QueryMonitoringFeature):
        """
        List Monitoring Feature

        :param query: The query information object
        :return: a generator object that yields :class:`~basin3d.core.models.MonitoringFeature`
            objects
        """
        synthesis_messages: List[str] = []

        feature_type = isinstance(query.feature_type,
                                  FeatureTypeEnum) and query.feature_type.value or query.feature_type
        if feature_type in USGSDataSourcePlugin.feature_types or feature_type is None:

            # Convert parent_features
            usgs_regions = []
            usgs_subbasins = []
            parent_features = []
            if query.parent_feature:
                for value in query.parent_feature:
                    parent_features.append(value)
                    if len(value) < 4:
                        usgs_regions.append(value)
                    elif len(value) == 8:
                        usgs_subbasins.append(value)

            if not feature_type or feature_type != FeatureTypeEnum.POINT:

                huc_text = self.get_hydrological_unit_codes(synthesis_messages=synthesis_messages)
                logger.debug(f"{self.__class__.__name__}.list url:{URL_USGS_HUC}")

                for json_obj in [o for o in iter_rdb_to_json(huc_text) if
                                 not parent_features or [p for p in parent_features if o["huc"].startswith(p)]]:

                    monitoring_feature = None
                    if (feature_type is None or feature_type == FeatureTypeEnum.REGION) and len(json_obj["huc"]) < 4:
                        monitoring_feature = self._load_huc_obj(json_obj, feature_type=FeatureTypeEnum.REGION)

                    elif (feature_type is None or feature_type == FeatureTypeEnum.SUBREGION) and len(
                            json_obj["huc"]) == 4:
                        monitoring_feature = self._load_huc_obj(json_obj, feature_type=FeatureTypeEnum.SUBREGION,
                                                                related_sampling_feature=json_obj["huc"][0:2],
                                                                related_sampling_feature_type=FeatureTypeEnum.REGION)

                    elif (feature_type is None or feature_type == FeatureTypeEnum.BASIN) and len(json_obj["huc"]) == 6:
                        monitoring_feature = self._load_huc_obj(json_obj, feature_type=FeatureTypeEnum.BASIN,
                                                                related_sampling_feature=json_obj["huc"][0:4],
                                                                related_sampling_feature_type=FeatureTypeEnum.SUBREGION)

                    elif (feature_type is None or feature_type == FeatureTypeEnum.SUBBASIN) and len(
                            json_obj["huc"]) == 8:
                        hucs = {json_obj["huc"][0:i] for i in range(2, 8, 2)}

                        # Filter by regions if it is set
                        if not usgs_regions or not hucs.isdisjoint(usgs_regions):
                            # This is a Cataloging Unit (See https://water.usgs.gov/GIS/huc_name.html)
                            monitoring_feature = self._load_huc_obj(
                                json_obj=json_obj, feature_type=FeatureTypeEnum.SUBBASIN,
                                description="{} Watershed: Drainage basin code is defined by the USGS State "
                                            "Office where the site is located.".format(json_obj["basin"]),
                                related_sampling_feature=json_obj["huc"][0:6],
                                related_sampling_feature_type=FeatureTypeEnum.BASIN)

                    else:
                        logger.debug("Ignoring HUC {}".format(json_obj["huc"]))

                    # Determine whether to yield the monitoring feature object
                    if monitoring_feature:
                        if query.monitoring_feature and json_obj['huc'] in query.monitoring_feature:
                            yield monitoring_feature
                        elif not query.monitoring_feature:
                            yield monitoring_feature

            else:
                base_url = '{}site/?{}={}&seriesCatalogOutput=true&outputDataTypeCd=iv,dv&siteStatus=all&format=rdb'
                # Points by id: USGS calls these sites
                if query.monitoring_feature is not None:
                    usgs_sites = ",".join(query.monitoring_feature)
                    url = base_url.format(self.datasource.location, 'sites', usgs_sites)
                else:
                    # Point by subbasin: USGS calls subbasin as huc (instead of sites) to retrieve all subbasins
                    usgs_subbasin = ",".join(usgs_subbasins)
                    url = base_url.format(self.datasource.location, 'huc', usgs_subbasin)

                # Filter by locations with data
                usgs_site_response = get_url(url)
                logger.debug(f"{self.__class__.__name__}.list url:{url}")

                if usgs_site_response and usgs_site_response.status_code == 200:
                    observed_properties, unique_sites = _parse_sites_response(usgs_site_response)
                    for v in unique_sites.values():
                        yield _load_point_obj(datasource=self, json_obj=v,
                                              observed_property_variables=observed_properties,
                                              synthesis_messages=synthesis_messages)
        else:
            synthesis_messages.append(f"Feature type {feature_type} not supported by {self.datasource.name}.")
            logger.warning(f"Feature type {feature_type} not supported by {self.datasource.name}.")

        return StopIteration(synthesis_messages)

    def get_hydrological_unit_codes(self, synthesis_messages):
        """Get the hydrological unit codes for USGS"""

        try:
            response = get_url(URL_USGS_HUC, timeout=0.5)
            if response.status_code == 200:
                return response.text
            else:
                synthesis_messages.append(f"Get failed for {URL_USGS_HUC} - Failing over to stored HUC codes")
        except requests.exceptions.ReadTimeout:
            synthesis_messages.append(f"Read Timeout for {URL_USGS_HUC} - Failing over to stored HUC codes")
            logger.warning(f"Read Timeout for {URL_USGS_HUC} - Failing over to stored HUC codes")
        except requests.exceptions.ConnectTimeout:
            synthesis_messages.append(f"Connection Timeout for {URL_USGS_HUC} - Failing over to stored HUC codes")
            logger.warning(f"Connection Timeout for {URL_USGS_HUC} - Failing over to stored HUC codes")

        return usgs_huc_codes.CONTENT

    def get(self, query: QueryMonitoringFeature):
        """ Get a single Monitoring Feature object

        :param query: The query info object with id specified
        :return: a :class:`basin3d.core.models.MonitoringFeature` object
        """
        # query.id will always be a string at this point with validation upstream, thus ignoring the type checking

        monitoring_feature_len = len(query.id)  # type: ignore[arg-type]
        if not query.feature_type:
            if monitoring_feature_len == 2:
                query.feature_type = FeatureTypeEnum.REGION
            elif monitoring_feature_len == 4:
                query.feature_type = FeatureTypeEnum.SUBREGION
            elif monitoring_feature_len == 6:
                query.feature_type = FeatureTypeEnum.BASIN
            elif monitoring_feature_len == 8:
                query.feature_type = FeatureTypeEnum.SUBBASIN
            else:
                query.feature_type = FeatureTypeEnum.POINT

        query.monitoring_feature = [query.id]  # type: ignore[list-item]

        for o in self.list(query=query):
            return o

        # An 8 character code can also be a point, Try that
        if monitoring_feature_len == 8:
            query.feature_type = FeatureTypeEnum.POINT
            for o in self.list(query=query):
                return o
        return None

    def _load_huc_obj(self, json_obj, feature_type, description=None,
                      related_sampling_feature=None, related_sampling_feature_type=None):
        """
        Transform USGS huc information to a :class:`~basin3d.core.models.MonitoringFeature` object

        :param json_obj: USGS Site Service location object
        :return: a serialized :class:`~basin3d.core.models.MonitoringFeature` object
        """
        if not description:
            description = "{}: {}".format(feature_type, json_obj["basin"])

        related_sampling_feature_complex = list()
        if related_sampling_feature:
            related_sampling_feature_complex = [RelatedSamplingFeature(
                self,
                related_sampling_feature=related_sampling_feature,
                related_sampling_feature_type=related_sampling_feature_type,
                role=RelatedSamplingFeature.ROLE_PARENT
            )]

        result = None
        if json_obj:
            result = MonitoringFeature(
                self, id=json_obj["huc"],
                name=json_obj["basin"],
                description=description,
                feature_type=feature_type,
                shape=SpatialSamplingShapes.SHAPE_SURFACE,
                coordinates=None,
                related_sampling_feature_complex=related_sampling_feature_complex,
                observed_properties=None)
        return result


class USGSMeasurementTimeseriesTVPObservationAccess(DataSourcePluginAccess):
    """
    USGS Daily Values Service: https://waterservices.usgs.gov/docs/dv-service/

    USGS Instantaneous Values Service: https://waterservices.usgs.gov/docs/instantaneous-values/

    Access for mapping USGS water services daily or instantaneous value data to
    :class:`basin3d.core.models.MeasurementTimeseriesTVPObservation` objects.

    Daily Value and Instantaneous Value Qualification Code (dv_rmk_cd)

    =============  =========  ================================================================================
    BASIN-3D Code  USGS Code  Description
    =============  =========  ================================================================================
    ESTIMATED      e          Value has been edited or estimated by USGS personnel and is write protected
    NOT_SUPPORTED  &          Value was computed from affected unit values
    ESTIMATED      E          Value was computed from estimated unit values.
    VALIDATED      A          Approved for publication -- Processing and review completed.
    UNVALIDATED    P          Provisional data subject to revision.
    NOT_SUPPORTED  <          The value is known to be less than reported value and is write protected.
    NOT_SUPPORTED  >          The value is known to be greater than reported value and is write protected.
    NOT_SUPPORTED  1          Value is write protected without any remark code to be printed
    NOT_SUPPORTED  2          Remark is write protected without any remark code to be printed
    NOT_SUPPORTED  _          No remark (blank)
    =============  =========  ================================================================================

    """

    synthesis_model_class = MeasurementTimeseriesTVPObservation

    def list(self, query: QueryMeasurementTimeseriesTVP):
        """
        List of Measurement Timeseries TVP Observation objects for USGS Daily Values or Instantaneous Values

        :param query: :class:`basin3d.core.schema.query.QueryMeasurementTimeseriesTVP`
        :return: a generator object that yields :class:`~basin3d.synthesis.models.measurement.MeasurementTimeseriesTVPObservation` objects
        """
        synthesis_messages = []
        feature_obj_dict = {}
        if not query.monitoring_feature:
            return None

        search_params = ",".join(query.monitoring_feature)

        base_url = '{}site/?{}={}&seriesCatalogOutput=true&outputDataTypeCd=iv,dv&siteStatus=all&format=rdb'
        url = base_url.format(self.datasource.location, 'sites', search_params)

        if len(search_params) < 3:
            url = base_url.format(self.datasource.location, 'huc', search_params)

        usgs_site_response = None
        try:
            usgs_site_response = get_url(url)
            logger.debug(f"{self.__class__.__name__}.list url:{url}")
        except Exception as e:
            synthesis_messages.append("Could not connect to USGS site info: {}".format(e))
            logger.warning("Could not connect to USGS site info: {}".format(e))
            logger.warning("url: ", url)
            synthesis_messages.append(f"Url: {url}")

        if usgs_site_response:
            # Get observed property variables and unique sites from parse_site_response
            observed_properties, sites = _parse_sites_response(usgs_site_response)
            for v in sites.values():
                if v["site_no"]:
                    feature_obj_dict[v["site_no"]] = v

        # Iterate over data objects returned
        for data_json in generator_usgs_measurement_timeseries_tvp_observation(self, query, synthesis_messages):
            unit_of_measurement = data_json["variable"]["unit"]['unitCode']
            timezone_offset = data_json["sourceInfo"]["timeZoneInfo"]["defaultTimeZone"]["zoneOffset"]

            # name has agency, sitecode, parameter id and stat code
            #   e.g. "USGS:385106106571000:00060:00003"
            _, feature_id, parameter, statistic = data_json["name"].split(":")

            if feature_id in feature_obj_dict.keys():
                monitoring_feature = _load_point_obj(
                    datasource=self,
                    json_obj=feature_obj_dict[feature_id],
                    observed_property_variables=observed_properties,
                    synthesis_messages=synthesis_messages)
            else:
                # ToDo: expand this to use the info in the data return
                # ToDo: log message
                monitoring_feature = None

            result_TVPs = []
            result_TVP_quality = []
            result_quality = set()
            has_filtered_data_points = 0

            if len(data_json["values"]) > 1:
                # Cannot think of the case in which response should have more than one values so adding a message to track it.
                synthesis_messages.append(
                    f'{feature_id} has more than one timeseries returned for {parameter}. Contact plugin developer if you see this message.')

            for values in data_json["values"]:
                # The original code checked what USGS reported for the qualities present in the timeseries. Instead will build from individual point qualities below.
                # result_quality.update(self.get_result_qualifiers(values["qualifier"]))

                for value in values["value"]:

                    if len(value['qualifiers']) > 1:
                        # ToDo: add some error messaging.
                        pass

                    # result_point_quality = self.map_result_quality(value['qualifiers'])
                    result_point_quality = value['qualifiers'][0]

                    if not query.result_quality or result_point_quality in query.result_quality:

                        # Get the broker parameter
                        try:
                            try:
                                data: Optional[float] = float(value['value'])
                                data_str = value['value']
                                # Hardcoded unit conversion for river discharge parameters
                                data, unit_of_measurement = convert_discharge(data, data_str, parameter, unit_of_measurement)

                                if data:
                                    result_quality.add(result_point_quality)
                                    result_TVPs.append(TimeValuePair(timestamp=value['dateTime'], value=data))
                                    result_TVP_quality.append(result_point_quality)

                                continue

                            except Exception as e:
                                synthesis_messages.append(f"Unit Conversion Issue: {str(e)}")
                                logger.error(str(e))

                        except Exception as e:
                            synthesis_messages.append(f"TimeValuePair ERROR: {str(e)}")
                            logger.error(e)

                    elif query.result_quality and result_point_quality not in query.result_quality:
                        has_filtered_data_points += 1

                if has_filtered_data_points > 0:
                    msg = f'{feature_id} - {parameter}: {str(has_filtered_data_points)} timestamps did not match data quality query.'
                    synthesis_messages.append(msg)

                if len(result_TVPs) == 0:
                    msg = f'{feature_id} had no valid data values for {parameter} that match the query.'
                    synthesis_messages.append(msg)
                    continue

            measurement_timeseries_tvp_observation = MeasurementTimeseriesTVPObservation(
                self,
                id=feature_id,  # FYI: this field is not unique and thus kinda useless
                unit_of_measurement=unit_of_measurement,
                feature_of_interest_type=FeatureTypeEnum.POINT,
                feature_of_interest=monitoring_feature,
                utc_offset=int(timezone_offset.split(":")[0]),
                result=ResultListTVP(plugin_access=self, value=result_TVPs, result_quality=result_TVP_quality),
                observed_property=parameter,
                result_quality=list(result_quality),
                aggregation_duration=query.aggregation_duration[0],
                time_reference_position=TimeMetadataMixin.TIME_REFERENCE_MIDDLE,
                statistic=statistic
            )

            yield measurement_timeseries_tvp_observation

        return StopIteration(synthesis_messages)


@basin3d_plugin
class USGSDataSourcePlugin(DataSourcePluginPoint):
    title = 'USGS Data Source Plugin'
    plugin_access_classes = (USGSMonitoringFeatureAccess, USGSMeasurementTimeseriesTVPObservationAccess)

    feature_types = ['POINT', 'REGION', 'BASIN', 'SUBREGION', 'SUBBASIN']

    class DataSourceMeta:
        """
        This is an internal metadata class for defining additional :class:`basin3d.core.models.DataSource`
        attributes.

        **Attributes:**
            - *id* - unique id short name
            - *name* - human friendly name (more descriptive)
            - *location* - resource location
            - *id_prefix* - id prefix to make model object ids unique across plugins
            - *credentials_format* - if the data source requires authentication, this is where the
                format of the stored credentials is defined.

        """
        # Data Source attributes
        id = 'USGS'  # unique id for the datasource
        location = 'https://waterservices.usgs.gov/nwis/'
        id_prefix = 'USGS'
        name = 'USGS'  # Human Friendly Data Source Name



CONTENT = """huc	basin
<!--10n	60s-->
01	New England
0101	St. John
010100	St. John
01010001	Upper St. John
01010002	Allagash
01010003	Fish
01010004	Aroostook
01010005	Meduxnekeag
0102	Penobscot
010200	Penobscot
01020001	West Branch Penobscot
01020002	East Branch Penobscot
01020003	Mattawamkeag
01020004	Piscataquis
01020005	Lower Penobscot
0103	Kennebec
010300	Kennebec
01030001	Upper Kennebec
01030002	Dead
01030003	Lower Kennebec
0104	Androscoggin
010400	Androscoggin
01040001	Upper Androscoggin
01040002	Lower Androscoggin
0105	Maine Coastal
010500	Maine Coastal
01050001	St. Croix
01050002	Maine Coastal
01050003	St. George-Sheepscot
0106	Saco
010600	Saco
01060001	Presumpscot
01060002	Saco
01060003	Piscataqua-Salmon Falls
0107	Merrimack
010700	Merrimack
01070001	Pemigewasset
01070002	Merrimack
01070003	Contoocook
01070004	Nashua
01070005	Concord
0108	Connecticut
010801	Upper Connecticut
01080101	Upper Connecticut
01080102	Passumpsic
01080103	Waits
01080104	Upper Connecticut-Mascoma
01080105	White
01080106	Black-Ottauquechee
01080107	West
010802	Lower Connecticut
01080201	Middle Connecticut
01080202	Miller
01080203	Deerfield
01080204	Chicopee
01080205	Lower Connecticut
01080206	Westfield
01080207	Farmington
0109	Massachusetts-Rhode Island Coastal
010900	Massachusetts-Rhode Island Coastal
01090001	Charles
01090002	Cape Cod
01090003	Blackstone
01090004	Narragansett
01090005	Pawcatuck-Wood
0110	Connecticut Coastal
011000	Connecticut Coastal
01100001	Quinebaug
01100002	Shetucket
01100003	Thames
01100004	Quinnipiac
01100005	Housatonic
01100006	Saugatuck
01100007	Long Island Sound
0111	St. Francois
011100	St. Francois
01110000	St. Francois
02	Mid Atlantic
0201	Richelieu
020100	Richelieu
02010001	Lake George
02010002	Otter
02010003	Winooski
02010004	Ausable
02010005	Lamoille
02010006	Great Chazy-Saranac
02010007	Missisquoi
0202	Upper Hudson
020200	Upper Hudson
02020001	Upper Hudson
02020002	Sacandaga
02020003	Hudson-Hoosic
02020004	Mohawk
02020005	Schoharie
02020006	Middle Hudson
02020007	Rondout
02020008	Hudson-Wappinger
0203	Lower Hudson-Long Island
020301	Lower Hudson
02030101	Lower Hudson
02030102	Bronx
02030103	Hackensack-Passaic
02030104	Sandy Hook-Staten Island
02030105	Raritan
020302	Long Island
02030201	Northern Long Island
02030202	Southern Long Island
0204	Delaware
020401	Upper Delaware
02040101	Upper Delaware
02040102	East Branch Delaware
02040103	Lackawaxen
02040104	Middle Delaware-Mongaup-Brodhead
02040105	Middle Delaware-Musconetcong
02040106	Lehigh
020402	Lower Delaware
02040201	Crosswicks-Neshaminy
02040202	Lower Delaware
02040203	Schuylkill
02040204	Delaware Bay
02040205	Brandywine-Christina
02040206	Cohansey-Maurice
02040207	Broadkill-Smyrna
020403	New Jersey Coastal
02040301	Mullica-Toms
02040302	Great Egg Harbor
0205	Susquehanna
020501	Upper Susquehanna
02050101	Upper Susquehanna
02050102	Chenango
02050103	Owego-Wappasening
02050104	Tioga
02050105	Chemung
02050106	Upper Susquehanna-Tunkhannock
02050107	Upper Susquehanna-Lackawanna
020502	West Branch Susquehanna
02050201	Upper West Branch Susquehanna
02050202	Sinnemahoning
02050203	Middle West Branch Susquehanna
02050204	Bald Eagle
02050205	Pine
02050206	Lower West Branch Susquehanna
020503	Lower Susquehanna
02050301	Lower Susquehanna-Penns
02050302	Upper Juniata
02050303	Raystown
02050304	Lower Juniata
02050305	Lower Susquehanna-Swatara
02050306	Lower Susquehanna
0206	Upper Chesapeake
020600	Upper Chesapeake
02060001	Upper Chesapeake Bay
02060002	Chester-Sassafras
02060003	Gunpowder-Patapsco
02060004	Severn
02060005	Choptank
02060006	Patuxent
02060007	Blackwater-Wicomico
02060008	Nanticoke
02060009	Pocomoke
02060010	Chincoteague
0207	Potomac
020700	Potomac
02070001	South Branch Potomac
02070002	North Branch Potomac
02070003	Cacapon-Town
02070004	Conococheague-Opequon
02070005	South Fork Shenandoah
02070006	North Fork Shenandoah
02070007	Shenandoah
02070008	Middle Potomac-Catoctin
02070009	Monocacy
02070010	Middle Potomac-Anacostia-Occoquan
02070011	Lower Potomac
0208	Lower Chesapeake
020801	Lower Chesapeake
02080101	Lower Chesapeake Bay
02080102	Great Wicomico-Piankatank
02080103	Rapidan-Upper Rappahannock
02080104	Lower Rappahannock
02080105	Mattaponi
02080106	Pamunkey
02080107	York
02080108	Lynnhaven-Poquoson
02080109	Western Lower Delmarva
02080110	Eastern Lower Delmarva
020802	James
02080201	Upper James
02080202	Maury
02080203	Middle James-Buffalo
02080204	Rivanna
02080205	Middle James-Willis
02080206	Lower James
02080207	Appomattox
02080208	Hampton Roads
03	South Atlantic-Gulf
0301	Chowan-Roanoke
030101	Roanoke
03010101	Upper Roanoke
03010102	Middle Roanoke
03010103	Upper Dan
03010104	Lower Dan
03010105	Banister
03010106	Roanoke Rapids
03010107	Lower Roanoke
030102	Albemarle-Chowan
03010201	Nottoway
03010202	Blackwater
03010203	Chowan
03010204	Meherrin
03010205	Albemarle
0302	Neuse-Pamlico
030201	Pamlico
03020101	Upper Tar
03020102	Fishing
03020103	Lower Tar
03020104	Pamlico
03020105	Pamlico Sound
03020106	Bogue-Core Sounds
030202	Neuse
03020201	Upper Neuse
03020202	Middle Neuse
03020203	Contentnea
03020204	Lower Neuse
0303	Cape Fear
030300	Cape Fear
03030001	New
03030002	Haw
03030003	Deep
03030004	Upper Cape Fear
03030005	Lower Cape Fear
03030006	Black
03030007	Northeast Cape Fear
0304	Pee Dee
030401	Upper Pee Dee
03040101	Upper Yadkin
03040102	South Yadkin
03040103	Lower Yadkin
03040104	Upper Pee Dee
03040105	Rocky
030402	Lower Pee Dee
03040201	Lower Pee Dee
03040202	Lynches
03040203	Lumber
03040204	Little Pee Dee
03040205	Black
03040206	Waccamaw
03040207	Carolina Coastal-Sampit
0305	Edisto-Santee
030501	Santee
03050101	Upper Catawba
03050102	South Fork Catawba
03050103	Lower Catawba
03050104	Wateree
03050105	Upper Broad
03050106	Lower Broad
03050107	Tyger
03050108	Enoree
03050109	Saluda
03050110	Congaree
03050111	Lake Marion
03050112	Santee
030502	Edisto-South Carolina Coastal
03050201	Cooper
03050202	South Carolina Coastal
03050203	North Fork Edisto
03050204	South Fork Edisto
03050205	Edisto
03050206	Four Hole Swamp
03050207	Salkehatchie
03050208	Broad-St. Helena
0306	Ogeechee-Savannah
030601	Savannah
03060101	Seneca
03060102	Tugaloo
03060103	Upper Savannah
03060104	Broad
03060105	Little
03060106	Middle Savannah
03060107	Stevens
03060108	Brier
03060109	Lower Savannah
030602	Ogeechee
03060201	Upper Ogeechee
03060202	Lower Ogeechee
03060203	Canoochee
03060204	Ogeechee Coastal
0307	Altamaha - St. Marys
030701	Altamaha
03070101	Upper Oconee
03070102	Lower Oconee
03070103	Upper Ocmulgee
03070104	Lower Ocmulgee
03070105	Little Ocmulgee
03070106	Altamaha
03070107	Ohoopee
030702	St. Marys - Satilla
03070201	Satilla
03070202	Little Satilla
03070203	Cumberland-St. Simons
03070204	St. Marys
03070205	Nassau
0308	St. Johns
030801	St. Johns
03080101	Upper St. Johns
03080102	Oklawaha
03080103	Lower St. Johns
030802	East Florida Coastal
03080201	Daytona - St. Augustine
03080202	Cape Canaveral
03080203	Vero Beach
0309	Southern Florida
030901	Kissimmee
03090101	Kissimmee
03090102	Northern Okeechobee Inflow
03090103	Western Okeechobee Inflow
030902	Southern Florida
03090201	Lake Okeechobee
03090202	Everglades
03090203	Florida Bay-Florida Keys
03090204	Big Cypress Swamp
03090205	Caloosahatchee
0310	Peace-Tampa Bay
031001	Peace
03100101	Peace
03100102	Myakka
03100103	Charlotte Harbor
031002	Tampa Bay
03100201	Sarasota Bay
03100202	Manatee
03100203	Little Manatee
03100204	Alafia
03100205	Hillsborough
03100206	Tampa Bay
03100207	Crystal-Pithlachascotee
03100208	Withlacoochee
0311	Suwannee
031101	Aucilla-Waccasassa
03110101	Waccasassa
03110102	Econfina-Steinhatchee
03110103	Aucilla
031102	Suwannee
03110201	Upper Suwannee
03110202	Alapaha
03110203	withlacoochee
03110204	Little
03110205	Lower Suwannee
03110206	Santa Fe
0312	Ochlockonee
031200	Ochlockonee. Georgia
03120001	Apalachee Bay-St. Marks
03120002	Upper Ochlockonee
03120003	Lower Ochlockonee
0313	Apalachicola
031300	Apalachicola
03130001	Upper Chattahoochee
03130002	Middle Chattahoochee-Lake Harding
03130003	Middle Chattahoochee-Walter F. George Reservoir
03130004	Lower Chattahoochee
03130005	Upper Flint
03130006	Middle Flint
03130007	Kinchafoonee-Muckalee
03130008	Lower Flint
03130009	Ichawaynochaway
03130010	Spring
03130011	Apalachicola
03130012	Chipola
03130013	New
03130014	Apalachicola Bay
0314	Choctawhatchee - Escambia
031401	Florida Panhandle Coastal
03140101	St. Andrew-St. Joseph Bays
03140102	Choctawhatchee Bay
03140103	Yellow
03140104	Blackwater
03140105	Pensacola Bay
03140106	Perdido
03140107	Perdido Bay
031402	Choctawhatchee
03140201	Upper Choctawhatchee
03140202	Pea
03140203	Lower Choctawhatchee
031403	Escambia
03140301	Upper Conecuh
03140302	Patsaliga
03140303	Sepulga
03140304	Lower Conecuh
03140305	Escambia
0315	Alabama
031501	Coosa-Tallapoosa
03150101	Conasauga
03150102	Coosawattee
03150103	Oostanaula
03150104	Etowah
03150105	Upper Coosa
03150106	Middle Coosa
03150107	Lower Coosa
03150108	Upper Tallapoosa
03150109	Middle Tallapoosa
03150110	Lower Tallapoosa
031502	Alabama
03150201	Upper Alabama
03150202	Cahaba
03150203	Middle Alabama
03150204	Lower Alabama
0316	Mobile - Tombigbee
031601	Black Warrior - Tombigbee
03160101	Upper Tombigbee
03160102	Town
03160103	Buttahatchee
03160104	Tibbee
03160105	Luxapallila
03160106	Middle Tombigbee-Lubbub
03160107	Sipsey
03160108	Noxubee
03160109	Mulberry
03160110	Sipsey Fork
03160111	Locust
03160112	Upper Black Warrior
03160113	Lower Black Warrior
031602	Mobile Bay- Tombigbee
03160201	Middle Tombigbee-Chickasaw
03160202	Sucarnoochee
03160203	Lower Tambigbee
03160204	Mobile - Tensaw
03160205	Mobile Bay
0317	Pascagoula
031700	Pascagoula. Mississippi
03170001	Chunky-Okatibbee
03170002	Upper Chickasawhay
03170003	Lower Chickasawhay
03170004	Upper Leaf
03170005	Lower Leaf
03170006	Pascagoula
03170007	Black
03170008	Escatawpa
03170009	Mississippi Coastal
0318	Pearl
031800	Pearl
03180001	Upper Pearl
03180002	Middle Pearl-Strong
03180003	Middle Pearl-Silver
03180004	Lower Pearl. Mississippi
03180005	Bogue Chitto
04	Great Lakes
0401	Western Lake Superior
040101	Northwestern Lake Superior
04010101	Baptism-Brule
04010102	Beaver-Lester
040102	St. Louis
04010201	St. Louis
04010202	Cloquet
040103	Southwestern Lake Superior
04010301	Beartrap-Nemadji
04010302	Bad-Montreal
0402	Southern Lake Superior-Lake Superior
040201	Southcentral Lake Superior
04020101	Black-Presque Isle
04020102	Ontonagon
04020103	Keweenaw Peninsula
04020104	Sturgeon
04020105	Dead-Kelsey
040202	Southeastern Lake Superior
04020201	Betsy-Chocolay
04020202	Tahquamenon
04020203	Waiska
040203	Lake Superior
04020300	Lake Superior
0403	Northwestern Lake Michigan
040301	Northwestern Lake Michigan
04030101	Manitowoc-Sheboygan
04030102	Door-Kewaunee
04030103	Duck-Pensaukee
04030104	Oconto
04030105	Peshtigo
04030106	Brule
04030107	Michigamme
04030108	Menominee
04030109	Cedar-Ford
04030110	Escanaba
04030111	Tacoosh-Whitefish
04030112	Fishdam-Sturgeon
040302	Fox
04030201	Upper Fox
04030202	Wolf
04030203	Lake Winnebago
04030204	Lower Fox
0404	Southwestern Lake Michigan
040400	Southwestern Lake Michigan
04040001	Little Calumet-Galien
04040002	Pike-Root
04040003	Milwaukee
0405	Southeastern Lake Michigan
040500	Southeastern Lake Michigan
04050001	St. Joseph
04050002	Black-Macatawa
04050003	Kalamazoo
04050004	Upper Grand
04050005	Maple
04050006	Lower Grand
04050007	Thornapple
0406	Northeastern Lake Michigan-Lake Michigan
040601	Northeastern Lake Michigan
04060101	Pere Marquette-White
04060102	Muskegon
04060103	Manistee
04060104	Betsie-Platte
04060105	Boardman-Charlevoix
04060106	Manistique
04060107	Brevoort-Millecoquins
040602	Lake Michigan
04060200	Lake Michigan
0407	Northwestern Lake Huron
040700	Northwestern Lake Huron
04070001	St. Marys
04070002	Carp-Pine
04070003	Lone Lake-Ocqueoc
04070004	Cheboygan
04070005	Black
04070006	Thunder Bay
04070007	Au Sable
0408	Southwestern Lake Huron-Lake Huron
040801	Southwestern Lake Huron
04080101	Au Gres-Rifle
04080102	Kawkawlin-Pine
04080103	Pigeon-Wiscoggin
04080104	Birch-Willow
040802	Saginaw
04080201	Tittabawassee
04080202	Pine
04080203	Shiawassee
04080204	Flint
04080205	Cass
04080206	Saginaw
040803	Lake Huron
04080300	Lake Huron
0409	St. Clair-Detroit
040900	St. Clair-Detroit
04090001	St. Clair
04090002	Lake St. Clair
04090003	Clinton
04090004	Detroit
04090005	Huron
0410	Western Lake Erie
041000	Western Lake Erie
04100001	Ottawa-Stony
04100002	Raisin
04100003	St. Joseph
04100004	St. Marys
04100005	Upper Maumee
04100006	Tiffin
04100007	Auglaize
04100008	Blanchard
04100009	Lower Maumee
04100010	Cedar-Portage
04100011	Sandusky
04100012	Huron-Vermilion
0411	Southern Lake Erie
041100	Southern Lake Erie
04110001	Black-Rocky
04110002	Cuyahoga
04110003	Ashtabula-Chagrin
04110004	Grand
0412	Eastern Lake Erie-Lake Erie
041201	Eastern Lake Erie
04120101	Chautauqua-Conneaut
04120102	Cattaraugus
04120103	Buffalo-Eighteenmile
04120104	Niagara
041202	Lake Erie
04120200	Lake Erie
0413	Southwestern Lake Ontario
041300	Southwestern Lake Ontario
04130001	Oak Orchard-Twelvemile
04130002	Upper Genesee
04130003	Lower Genesee
0414	Southeastern Lake Ontario
041401	Southeastern Lake Ontario
04140101	Irondequoit-Ninemile
04140102	Salmon-Sandy
041402	Oswego
04140201	Seneca
04140202	Oneida
04140203	Oswego
0415	Northeastern Lake Ontario-Lake Ontario-St. Lawrence
041501	Northeastern Lake Ontario
04150101	Black
04150102	Chaumont-Perch
041502	Lake Ontario
04150200	Lake Ontario
041503	St. Lawrence
04150301	Upper St. Lawrence
04150302	Oswegatchie
04150303	Indian
04150304	Grass
04150305	Raquette
04150306	St. Regis
04150307	English-Salmon
05	Ohio
0501	Allegheny
050100	Allegheny
05010001	Upper Allegheny
05010002	Conewango
05010003	Middle Allegheny-Tionesta
05010004	French
05010005	Clarion
05010006	Middle Allegheny-Redbank
05010007	Conemaugh
05010008	Kiskiminetas
05010009	Lower Allegheny
0502	Monongahela
050200	Monongahela
05020001	Tygart Valley
05020002	West Fork
05020003	Upper Monongahela
05020004	Cheat
05020005	Lower Monongahela
05020006	Youghiogheny
0503	Upper Ohio
050301	Upper Ohio-Beaver
05030101	Upper Ohio
05030102	Shenango
05030103	Mahoning
05030104	Beaver
05030105	Connoquenessing
05030106	Upper Ohio-Wheeling
050302	Upper Ohio-Little Kanawha
05030201	Little Muskingum-Middle Island
05030202	Upper Ohio-Shade
05030203	Little Kanawha
05030204	Hocking
0504	Muskingum
050400	Muskingum
05040001	Tuscarawas
05040002	Mohican
05040003	Walhonding
05040004	Muskingum
05040005	Wills
05040006	Licking
0505	Kanawha
050500	Kanawha
05050001	Upper New
05050002	Middle New
05050003	Greenbrier
05050004	Lower New
05050005	Gauley
05050006	Upper Kanawha
05050007	Elk
05050008	Lower Kanawha
05050009	Coal
0506	Scioto
050600	Scioto
05060001	Upper Scioto
05060002	Lower Scioto
05060003	Paint
0507	Big Sandy-Guyandotte
050701	Guyandotte
05070101	Upper Guyandotte
05070102	Lower Guyandotte
050702	Big Sandy
05070201	Tug
05070202	Upper Levisa
05070203	Lower Levisa
05070204	Big Sandy
0508	Great Miami
050800	Great Miami
05080001	Upper Great Miami
05080002	Lower Great Miami
05080003	Whitewater
0509	Middle Ohio
050901	Middle Ohio-Raccoon
05090101	Raccoon-Symmes
05090102	Twelvepole
05090103	Little Scioto-Tygarts
05090104	Little Sandy
050902	Middle Ohio-Little Miami
05090201	Ohio Brush-Whiteoak
05090202	Little Miami
05090203	Middle Ohio-Laughery
0510	Kentucky-Licking
051001	Licking
05100101	Licking
05100102	South Fork Licking
051002	Kentucky
05100201	North Fork Kentucky
05100202	Middle Fork Kentucky
05100203	South Fork Kentucky
05100204	Upper Kentucky
05100205	Lower Kentucky
0511	Green
051100	Green
05110001	Upper Green
05110002	Barren
05110003	Middle Green
05110004	Rough
05110005	Lower Green
05110006	Pond
0512	Wabash
051201	Wabash
05120101	Upper Wabash
05120102	Salamonie
05120103	Mississinewa
05120104	Eel
05120105	Middle Wabash-Deer
05120106	Tippecanoe
05120107	Wildcat
05120108	Middle Wabash-Little Vermilion
05120109	Vermilion
05120110	Sugar
05120111	Middle Wabash-Busseron
05120112	Embarras
05120113	Lower Wabash
05120114	Little Wabash
05120115	Skillet
051202	Patoka-White
05120201	Upper White
05120202	Lower White
05120203	Eel
05120204	Driftwood
05120205	Flatrock-Haw
05120206	Upper East Fork White
05120207	Muscatatuck
05120208	Lower East Fork White
05120209	Patoka
0513	Cumberland
051301	Upper Cumberland
05130101	Upper Cumberland
05130102	Rockcastle
05130103	Upper Cumberland-Lake Cumberland
05130104	South Fork Cumberland
05130105	Obey
05130106	Upper Cumberland-Cordell Hull
05130107	Collins
05130108	Caney
051302	Lower Cumberland
05130201	Lower Cumberland-Old Hickory Lake
05130202	Lower Cumberland-Sycamore
05130203	Stones
05130204	Harpeth
05130205	Lower Cumberland
05130206	Red
0514	Lower Ohio
051401	Lower Ohio-Salt
05140101	Silver-Little Kentucky
05140102	Salt
05140103	Rolling Fork
05140104	Blue-Sinking
051402	Lower Ohio
05140201	Lower Ohio-Little Pigeon
05140202	Highland-Pigeon
05140203	Lower Ohio-Bay
05140204	Saline
05140205	Tradewater
05140206	Lower Ohio
06	Tennessee
0601	Upper Tennessee
060101	French Broad-Holston
06010101	North Fork Holston
06010102	South Fork Holston
06010103	Watauga
06010104	Holston
06010105	Upper French Broad
06010106	Pigeon
06010107	Lower French Broad
06010108	Nolichucky
060102	Upper Tennessee
06010201	Watts Bar Lake
06010202	Upper Little Tennessee
06010203	Tuckasegee
06010204	Lower Little Tennessee
06010205	Upper Clinch
06010206	Powell
06010207	Lower Clinch
06010208	Emory
0602	Middle Tennessee-Hiwassee
060200	Middle Tennessee-Hiwassee
06020001	Middle Tennessee-Chickamauga
06020002	Hiwassee
06020003	Ocoee
06020004	Sequatchie
0603	Middle Tennessee-Elk
060300	Middle Tennessee-Elk
06030001	Guntersville Lake
06030002	Wheeler Lake
06030003	Upper Elk
06030004	Lower Elk
06030005	Pickwick Lake
06030006	Bear
0604	Lower Tennessee
060400	Lower Tennessee
06040001	Lower Tennessee-Beech
06040002	Upper Duck
06040003	Lower Duck
06040004	Buffalo
06040005	Kentucky Lake
06040006	Lower Tennessee
07	Upper Mississippi
0701	Mississippi Headwaters
070101	Mississippi Headwaters
07010101	Mississippi Headwaters
07010102	Leech Lake
07010103	Prairie-Willow
07010104	Elk-Nokasippi
07010105	Pine
07010106	Crow Wing
07010107	Redeye
07010108	Long Prairie
070102	Upper Mississippi-Crow-Rum
07010201	Platte-Spunk
07010202	Sauk
07010203	Clearwater-Elk
07010204	Crow
07010205	South Fork Crow
07010206	Twin Cities
07010207	Rum
0702	Minnesota
070200	Minnesota
07020001	Upper Minnesota
07020002	Pomme De Terre
07020003	Lac Qui Parle
07020004	Hawk-Yellow Medicine
07020005	Chippewa
07020006	Redwood
07020007	Middle Minnesota
07020008	Cottonwood
07020009	Blue Earth
07020010	Watonwan
07020011	Le Sueur
07020012	Lower Minnesota
0703	St. Croix
070300	St. Croix
07030001	Upper St. Croix
07030002	Namekagon
07030003	Kettle
07030004	Snake
07030005	Lower St. Croix
0704	Upper Mississippi-Black-Root
070400	Upper Mississippi-Black-Root
07040001	Rush-Vermillion
07040002	Cannon
07040003	Buffalo-Whitewater
07040004	Zumbro
07040005	Trempealeau
07040006	La Crosse-Pine
07040007	Black
07040008	Root
0705	Chippewa
070500	Chippewa
07050001	Upper Chippewa
07050002	Flambeau
07050003	South Fork Flambeau
07050004	Jump
07050005	Lower Chippewa
07050006	Eau Claire
07050007	Red Cedar
0706	Upper Mississippi-Maquoketa-Plum
070600	Upper Mississippi-Maquoketa-Plum
07060001	Coon-Yellow
07060002	Upper Iowa
07060003	Grant-Little Maquoketa
07060004	Turkey
07060005	Apple-Plum
07060006	Maquoketa
0707	Wisconsin
070700	Wisconsin
07070001	Upper Wisconsin
07070002	Lake Dubay
07070003	Castle Rock
07070004	Baraboo
07070005	Lower Wisconsin
07070006	Kickapoo
0708	Upper Mississippi-Iowa-Skunk-Wapsipinicon
070801	Upper Mississippi-Skunk-Wapsipinicon
07080101	Copperas-Duck
07080102	Upper Wapsipinicon
07080103	Lower Wapsipinicon
07080104	Flint-Henderson
07080105	South Skunk
07080106	North Skunk
07080107	Skunk
070802	Iowa
07080201	Upper Cedar
07080202	Shell Rock
07080203	Winnebago
07080204	West Fork Cedar
07080205	Middle Cedar
07080206	Lower Cedar
07080207	Upper Iowa
07080208	Middle Iowa
07080209	Lower Iowa
0709	Rock
070900	Rock
07090001	Upper Rock
07090002	Crawfish
07090003	Pecatonica
07090004	Sugar
07090005	Lower Rock
07090006	Kishwaukee
07090007	Green
0710	Des Moines
071000	Des Moines
07100001	Des Moines Headwaters
07100002	Upper Des Moines
07100003	East Fork Des Moines
07100004	Middle Des Moines
07100005	Boone
07100006	North Raccoon
07100007	South Raccoon
07100008	Lake Red Rock
07100009	Lower Des Moines
0711	Upper Mississippi-Salt
071100	Upper Mississippi-Salt
07110001	Bear-Wyaconda
07110002	North Fabius
07110003	South Fabius
07110004	The Sny
07110005	North Fork Salt
07110006	South Fork Salt
07110007	Salt
07110008	Cuivre
07110009	Peruque-Piasa
0712	Upper Illinois
071200	Upper Illinois
07120001	Kankakee
07120002	Iroquois
07120003	Chicago
07120004	Des Plaines
07120005	Upper Illinois
07120006	Upper Fox
07120007	Lower Fox
0713	Lower Illinois
071300	Lower Illinois
07130001	Lower Illinois-Senachwine Lake
07130002	Vermilion
07130003	Lower Illinois-Lake Chautauqua
07130004	Mackinaw
07130005	Spoon
07130006	Upper Sangamon
07130007	South Fork Sangamon
07130008	Lower Sangamon
07130009	Salt
07130010	La Moine
07130011	Lower Illinois
07130012	Macoupin
0714	Upper Mississippi-Kaskaskia-Meramec
071401	Upper Mississippi-Meramec
07140101	Cahokia-Joachim
07140102	Meramec
07140103	Bourbeuse
07140104	Big
07140105	Upper Mississippi-Cape Girardeau
07140106	Big Muddy
07140107	Whitewater
07140108	Cache
071402	Kaskaskia
07140201	Upper Kaskaskia
07140202	Middle Kaskaskia
07140203	Shoal
07140204	Lower Kaskaskia
08	Lower Mississippi
0801	Lower Mississippi-Hatchie
080101	Lower Mississippi-Memphis
08010100	Lower Mississippi-Memphis
080102	Hatchie-Obion
08010201	Bayou De Chien-Mayfield
08010202	Obion
08010203	South Fork Obion
08010204	North Fork Forked Deer
08010205	South Fork Forked Deer
08010206	Forked Deer
08010207	Upper Hatchie
08010208	Lower Hatchie
08010209	Loosahatchie
08010210	Wolf
08010211	Horn Lake-Nonconnah
0802	Lower Mississippi - St. Francis
080201	Lower Mississippi-Helena
08020100	Lower Mississippi-Helena
080202	St. Francis
08020201	New Madrid-St. Johns
08020202	Upper St. Francis
08020203	Lower St. Francis
08020204	Little River Ditches
08020205	L'anguille
080203	Lower White
08020301	Lower White-Bayou Des Arc
08020302	Cache
08020303	Lower White
08020304	Big
080204	Lower Arkansas
08020401	Lower Arkansas
08020402	Bayou Meto
0803	Lower Mississippi - Yazoo
080301	Lower Mississippi-Greenville
08030100	Lower Mississippi-Greenville
080302	Yazoo
08030201	Little Tallahatchie
08030202	Tallahatchie
08030203	Yocona
08030204	Coldwater
08030205	Yalobusha
08030206	Upper Yazoo
08030207	Big Sunflower
08030208	Lower Yazoo
08030209	Deer-Steele
0804	Lower Red - Ouachita
080401	Upper Ouachita
08040101	Ouachita Headwaters
08040102	Upper Ouachita
08040103	Little Missouri
080402	Lower Ouachita
08040201	Lower Ouachita-Smackover
08040202	Lower Ouachita-Bayou De Loutre
08040203	Upper Saline
08040204	Lower Saline
08040205	Bayou Bartholomew
08040206	Bayou D'arbonne
08040207	Lower Ouachita
080403	Lower Red
08040301	Lower Red
08040302	Castor
08040303	Dugdemona
08040304	Little
08040305	Black
08040306	Bayou Cocodrie
0805	Boeuf-Tensas
080500	Boeuf-Tensas
08050001	Boeuf
08050002	Bayou Macon
08050003	Tensas
0806	Lower Mississippi - Big Black
080601	Lower Mississippi-Natchez
08060100	Lower Mississippi-Natchez
080602	Big Black - Homochitto
08060201	Upper Big Black
08060202	Lower Big Black
08060203	Bayou Pierre
08060204	Coles Creek
08060205	Homochitto
08060206	Buffalo
0807	Lower Mississippi-Lake Maurepas
080701	Lower Mississippi-Baton Rouge
08070100	Lower Mississippi-Baton Rouge
080702	Lake Maurepas
08070201	Bayou Sara-Thompson
08070202	Amite
08070203	Tickfaw
08070204	Lake Maurepas
08070205	Tangipahoa
080703	Lower Grand
08070300	Lower Grand
0808	Louisiana Coastal
080801	Atchafalaya - Vermilion
08080101	Atchafalaya
08080102	Bayou Teche
08080103	Vermilion
080802	Calcasieu - Mermentau
08080201	Mermentau Headwaters
08080202	Mermentau
08080203	Upper Calcasieu
08080204	Whisky Chitto
08080205	West Fork Calcasieu
08080206	Lower Calcasieu
0809	Lower Mississippi
080901	Lower Mississippi-New Orleans
08090100	Lower Mississippi-New Orleans
080902	Lake Pontchartrain
08090201	Liberty Bayou-Tchefuncta
08090202	Lake Pontchartrain
08090203	Eastern Louisiana Coastal
080903	Central Louisiana Coastal
08090301	East Central Louisiana Coastal
08090302	West Central Louisiana Coastal
09	Souris-Red-Rainy
0901	Souris
090100	Souris
09010001	Upper Souris
09010002	Des Lacs
09010003	Lower Souris
09010004	Willow
09010005	Deep
0902	Red
090201	Upper Red
09020101	Bois De Sioux
09020102	Mustinka
09020103	Otter Tail
09020104	Upper Red
09020105	Western Wild Rice
09020106	Buffalo
09020107	Elm-Marsh
09020108	Eastern Wild Rice
09020109	Goose
090202	Devils Lake-Sheyenne
09020201	Devils Lake
09020202	Upper Sheyenne
09020203	Middle Sheyenne
09020204	Lower Sheyenne
09020205	Maple
090203	Lower Red
09020301	Sandhill-Wilson
09020302	Red Lakes
09020303	Red Lake
09020304	Thief
09020305	Clearwater
09020306	Grand Marais-Red
09020307	Turtle
09020308	Forest
09020309	Snake
09020310	Park
09020311	Lower Red
09020312	Two Rivers
09020313	Pembina
09020314	Roseau
0903	Rainy
090300	Rainy
09030001	Rainy Headwaters
09030002	Vermilion
09030003	Rainy Lake
09030004	Upper Rainy
09030005	Little Fork
09030006	Big Fork
09030007	Rapid
09030008	Lower Rainy
09030009	Lake of the Woods
10	Missouri
1001	Saskatchewan
100100	Saskatchewan
10010001	Belly
10010002	St. Mary
1002	Missouri Headwaters
100200	Missouri Headwaters
10020001	Red Rock
10020002	Beaverhead
10020003	Ruby
10020004	Big Hole
10020005	Jefferson
10020006	Boulder
10020007	Madison
10020008	Gallatin
1003	Missouri-Marias
100301	Upper Missouri
10030101	Upper Missouri
10030102	Upper Missouri-Dearborn
10030103	Smith
10030104	Sun
10030105	Belt
100302	Marias
10030201	Two Medicine
10030202	Cut Bank
10030203	Marias
10030204	Willow
10030205	Teton
1004	Missouri-Musselshell
100401	Fort Peck Lake
10040101	Bullwhacker-Dog
10040102	Arrow
10040103	Judith
10040104	Fort Peck Reservoir
10040105	Big Dry
10040106	Little Dry
100402	Musselshell
10040201	Upper Musselshell
10040202	Middle Musselshell
10040203	Flatwillow
10040204	Box Elder
10040205	Lower Musselshell
1005	Milk
100500	Milk
10050001	Milk Headwaters
10050002	Upper Milk
10050003	Wild Horse Lake
10050004	Middle Milk
10050005	Big Sandy
10050006	Sage
10050007	Lodge
10050008	Battle
10050009	Peoples
10050010	Cottonwood
10050011	Whitewater
10050012	Lower Milk
10050013	Frenchman
10050014	Beaver
10050015	Rock
10050016	Porcupine
1006	Missouri-Poplar
100600	Missouri-Poplar
10060001	Prarie Elk-Wolf
10060002	Redwater
10060003	Poplar
10060004	West Fork Poplar
10060005	Charlie-Little Muddy
10060006	Big Muddy
10060007	Brush Lake closed basin
1007	Upper Yellowstone
100700	Upper Yellowstone
10070001	Yellowstone Headwaters
10070002	Upper Yellowstone
10070003	Shields
10070004	Upper Yellowstone-Lake Basin
10070005	Stillwater
10070006	Clarks Fork Yellowstone
10070007	Upper Yellowstone-Pompeys Pillar
10070008	Pryor
1008	Big Horn
100800	Big Horn
10080001	Upper Wind
10080002	Little Wind
10080003	Popo Agie
10080004	Muskrat
10080005	Lower Wind
10080006	Badwater
10080007	Upper Bighorn
10080008	Nowood
10080009	Greybull
10080010	Big Horn Lake
10080011	Dry
10080012	North Fork Shoshone
10080013	South Fork Shoshone
10080014	Shoshone
10080015	Lower Bighorn
10080016	Little Bighorn
1009	Powder-Tongue
100901	Tongue
10090101	Upper Tongue
10090102	Lower Tongue
100902	Powder
10090201	Middle Fork Powder
10090202	Upper Powder
10090203	South Fork Powder
10090204	Salt
10090205	Crazy Woman
10090206	Clear
10090207	Middle Powder
10090208	Little Powder
10090209	Lower Powder
10090210	Mizpah
1010	Lower Yellowstone
101000	Lower Yellowstone
10100001	Lower Yellowstone-Sunday
10100002	Big Porcupine
10100003	Rosebud
10100004	Lower Yellowstone
10100005	O'fallon
1011	Missouri-Little Missouri
101101	Lake Sakakawea
10110101	Lake Sakakawea
10110102	Little Muddy
101102	Little Missouri
10110201	Upper Little Missouri
10110202	Boxelder
10110203	Middle Little Missouri
10110204	Beaver
10110205	Lower Little Missouri
1012	Cheyenne
101201	Cheyenne
10120101	Antelope
10120102	Dry Fork Cheyenne
10120103	Upper Cheyenne
10120104	Lance
10120105	Lightning
10120106	Angostura Reservoir
10120107	Beaver
10120108	Hat
10120109	Middle Cheyenne-Spring
10120110	Rapid
10120111	Middle Cheyenne-Elk
10120112	Lower Cheyenne
10120113	Cherry
101202	Belle Fourche
10120201	Upper Belle Fourche
10120202	Lower Belle Fourche
10120203	Redwater
1013	Missouri-Oahe
101301	Lake Oahe
10130101	Painted Woods-Square Butte
10130102	Upper Lake Oahe
10130103	Apple
10130104	Beaver
10130105	Lower Lake Oahe
10130106	West Missouri Coteau
101302	Cannonball-Heart-Knife
10130201	Knife
10130202	Upper Heart
10130203	Lower Heart
10130204	Upper Cannonball
10130205	Cedar
10130206	Lower Cannonball
101303	Grand-Moreau
10130301	North Fork Grand
10130302	South Fork Grand
10130303	Grand
10130304	South Fork Moreau
10130305	Upper Moreau
10130306	Lower Moreau
1014	Missouri-White
101401	Fort Randall Reservoir
10140101	Fort Randall Reservoir
10140102	Bad
10140103	Medicine Knoll
10140104	Medicine
10140105	Crow
101402	White
10140201	Upper White
10140202	Middle White
10140203	Little White
10140204	Lower White
1015	Niobrara
101500	Niobrara
10150001	Ponca
10150002	Niobrara Headwaters
10150003	Upper Niobrara
10150004	Middle Niobrara
10150005	Snake
10150006	Keya Paha
10150007	Lower Niobrara
1016	James
101600	James
10160001	James Headwaters
10160002	Pipestem
10160003	Upper James
10160004	Elm
10160005	Mud
10160006	Middle James
10160007	East Missouri Coteau
10160008	Snake
10160009	Turtle
10160010	North Big Sioux Coteau
10160011	Lower James
1017	Missouri-Big Sioux
101701	Lewis and Clark Lake
10170101	Lewis and Clark Lake
10170102	Vermillion
10170103	South Big Sioux Coteau
101702	Big Sioux
10170201	Middle Big Sioux Coteau
10170202	Upper Big Sioux
10170203	Lower Big Sioux
10170204	Rock
1018	North Platte
101800	North Platte
10180001	North Platte Headwaters
10180002	Upper North Platte
10180003	Pathfinder-Seminoe Reservoirs
10180004	Medicine Bow
10180005	Little Medicine Bow
10180006	Sweetwater
10180007	Middle North Platte-Casper
10180008	Glendo Reservoir
10180009	Middle North Platte-Scotts Bluff
10180010	Upper Laramie
10180011	Lower Laramie
10180012	Horse
10180013	Pumpkin
10180014	Lower North Platte
1019	South Platte
101900	South Platte
10190001	South Platte Headwaters
10190002	Upper South Platte
10190003	Middle South Platte-Cherry Creek
10190004	Clear
10190005	St. Vrain
10190006	Big Thompson
10190007	Cache La Poudre
10190008	Lone Tree-Owl
10190009	Crow
10190010	Kiowa
10190011	Bijou
10190012	Middle South Platte-Sterling
10190013	Beaver
10190014	Pawnee
10190015	Upper Lodgepole
10190016	Lower Lodgepole
10190017	Sidney Draw
10190018	Lower South Platte
1020	Platte
102001	Middle Platte
10200101	Middle Platte-Buffalo
10200102	Wood
10200103	Middle Platte-Prairie
102002	Lower Platte
10200201	Lower Platte-Shell
10200202	Lower Platte
10200203	Salt
1021	Loup
102100	Loup
10210001	Upper Middle Loup
10210002	Dismal
10210003	Lower Middle Loup
10210004	South Loup
10210005	Mud
10210006	Upper North Loup
10210007	Lower North Loup
10210008	Calamus
10210009	Loup
10210010	Cedar
1022	Elkhorn
102200	Elkhorn
10220001	Upper Elkhorn
10220002	North Fork Elkhorn
10220003	Lower Elkhorn
10220004	Logan
1023	Missouri-Little Sioux
102300	Missouri-Little Sioux
10230001	Blackbird-Soldier
10230002	Floyd
10230003	Little Sioux
10230004	Monona-Harrison Ditch
10230005	Maple
10230006	Big Papillion-Mosquito
10230007	Boyer
1024	Missouri-Nishnabotna
102400	Missouri-Nishnabotna
10240001	Keg-Weeping Water
10240002	West Nishnabotna
10240003	East Nishnabotna
10240004	Nishnabotna
10240005	Tarkio-Wolf
10240006	Little Nemaha
10240007	South Fork Big Nemaha
10240008	Big Nemaha
10240009	West Nodaway
10240010	Nodaway
10240011	Independence-Sugar
10240012	Platte
10240013	One Hundred and Two
1025	Republican
102500	Republican
10250001	Arikaree
10250002	North Fork Republican
10250003	South Fork Republican
10250004	Upper Republican
10250005	Frenchman
10250006	Stinking Water
10250007	Red Willow
10250008	Medicine
10250009	Harlan County Reservoir
10250010	Upper Sappa
10250011	Lower Sappa
10250012	South Fork Beaver
10250013	Little Beaver
10250014	Beaver
10250015	Prairie Dog
10250016	Middle Republican
10250017	Lower Republican
1026	Smoky Hill
102600	Smoky Hill
10260001	Smoky Hill Headwaters
10260002	North Fork Smoky Hill
10260003	Upper Smoky Hill
10260004	Ladder
10260005	Hackberry
10260006	Middle Smoky Hill
10260007	Big
10260008	Lower Smoky Hill
10260009	Upper Saline
10260010	Lower Saline
10260011	Upper North Fork Solomon
10260012	Lower North Fork Solomon
10260013	Upper South Fork Solomon
10260014	Lower South Fork Solomon
10260015	Solomon
1027	Kansas
102701	Kansas
10270101	Upper Kansas
10270102	Middle Kansas
10270103	Delaware
10270104	Lower Kansas
102702	Big Blue
10270201	Upper Big Blue
10270202	Middle Big Blue
10270203	West Fork Big Blue
10270204	Turkey
10270205	Lower Big Blue
10270206	Upper Little Blue
10270207	Lower Little Blue
1028	Chariton-Grand
102801	Grand
10280101	Upper Grand
10280102	Thompson
10280103	Lower Grand
102802	Chariton
10280201	Upper Chariton
10280202	Lower Chariton
10280203	Little Chariton
1029	Gasconade-Osage
102901	Osage
10290101	Upper Marais Des Cygnes
10290102	Lower Marais Des Cygnes
10290103	Little Osage
10290104	Marmaton
10290105	Harry S. Missouri
10290106	Sac
10290107	Pomme De Terre
10290108	South Grand
10290109	Lake of the Ozarks
10290110	Niangua
10290111	Lower Osage
102902	Gasconade
10290201	Upper Gasconade
10290202	Big Piney
10290203	Lower Gasconade
1030	Lower Missouri
103001	Lower Missouri-Blackwater
10300101	Lower Missouri-Crooked
10300102	Lower Missouri-Moreau
10300103	Lamine
10300104	Blackwater
103002	Lower Missouri
10300200	Lower Missouri
11	Arkansas-White-Red
1101	Upper White
110100	Upper White
11010001	Beaver Reservoir
11010002	James
11010003	Bull Shoals Lake
11010004	Middle White
11010005	Buffalo
11010006	North Fork White
11010007	Upper Black
11010008	Current
11010009	Lower Black
11010010	Spring
11010011	Eleven Point
11010012	Strawberry
11010013	Upper White-Village
11010014	Little Red
1102	Upper Arkansas
110200	Upper Arkansas
11020001	Arkansas Headwaters
11020002	Upper Arkansas
11020003	Fountain
11020004	Chico
11020005	Upper Arkansas-Lake Meredith
11020006	Huerfano
11020007	Apishapa
11020008	Horse
11020009	Upper Arkansas-John Martin
11020010	Purgatoire
11020011	Big Sandy
11020012	Rush
11020013	Two Butte
1103	Middle Arkansas
110300	Middle Arkansas
11030001	Middle Arkansas-Lake Mckinney
11030002	Whitewoman
11030003	Arkansas-Dodge City
11030004	Coon-Pickerel
11030005	Pawnee
11030006	Buckner
11030007	Upper Walnut Creek
11030008	Lower Walnut Creek
11030009	Rattlesnake
11030010	Gar-Peace
11030011	Cow
11030012	Little Arkansas
11030013	Middle Arkansas-Slate
11030014	North Fork Ninnescah
11030015	South Fork Ninnescah
11030016	Ninnescah
11030017	Upper Walnut River
11030018	Lower Walnut River
1104	Upper Cimarron
110400	Upper Cimarron
11040001	Cimarron headwaters
11040002	Upper Cimarron
11040003	North Fork Cimarron
11040004	Sand Arroyo
11040005	Bear
11040006	Upper Cimarron-Liberal
11040007	Crooked
11040008	Upper Cimarron-Bluff
1105	Lower Cimarron
110500	Lower Cimarron
11050001	Lower Cimarron-Eagle Chief
11050002	Lower Cimarron-Skeleton
11050003	Lower Cimarron
1106	Arkansas - Keystone
110600	Arkansas - Keystone
11060001	Kaw Lake
11060002	Upper Salt Fork Arkansas
11060003	Medicine Lodge
11060004	Lower Salt Fork Arkansas
11060005	Chikaskia
11060006	Black Bear-Red Rock
1107	Neosho - Verdigris
110701	Verdigris
11070101	Upper Verdigris
11070102	Fall
11070103	Middle Verdigris
11070104	Elk
11070105	Lower Verdigris
11070106	Caney
11070107	Bird
110702	Neosho
11070201	Neosho headwaters
11070202	Upper Cottonwood
11070203	Lower Cottonwood
11070204	Upper Neosho
11070205	Middle Neosho
11070206	Lake O' the Cherokees
11070207	Spring
11070208	Elk
11070209	Lower Neosho
1108	Upper Canadian
110800	Upper Canadian
11080001	Canadian headwaters
11080002	Cimarron
11080003	Upper Canadian
11080004	Mora
11080005	Conchas
11080006	Upper Canadian-Ute Reservoir
11080007	Ute
11080008	Revuelto
1109	Lower Canadian
110901	Middle Canadian
11090101	Middle Canadian-Trujillo
11090102	Punta De Agua
11090103	Rita Blanca
11090104	Carrizo
11090105	Lake Meredith
11090106	Middle Canadian-Spring
110902	Lower Canadian
11090201	Lower Canadian-Deer
11090202	Lower Canadian-Walnut
11090203	Little
11090204	Lower Canadian
1110	North Canadian
111001	Upper Beaver
11100101	Upper Beaver
11100102	Middle Beaver
11100103	Coldwater
11100104	Palo Duro
111002	Lower Beaver
11100201	Lower Beaver
11100202	Upper Wolf
11100203	Lower Wolf
111003	Lower North Canadian
11100301	Middle North Canadian
11100302	Lower North Canadian
11100303	Deep Fork
1111	Lower Arkansas
111101	Robert S. Kerr Reservoir
11110101	Polecat-Snake
11110102	Dirty-Greenleaf
11110103	Illinois
11110104	Robert S. Kerr Reservoir
11110105	Poteau
111102	Lower Arkansas-Fourche La Fave
11110201	Frog-Mulberry
11110202	Dardanelle Reservoir
11110203	Lake Conway-Point Remove
11110204	Petit Jean
11110205	Cadron
11110206	Fourche La Fave
11110207	Lower Arkansas-Maumelle
1112	Red headwaters
111201	Prairie Dog Town Fork Red
11120101	Tierra Blanca
11120102	Palo Duro
11120103	Upper Prairie Dog Town Fork Red
11120104	Tule
11120105	Lower Prairie Dog Town Fork Red
111202	Salt Fork Red
11120201	Upper Salt Fork Red
11120202	Lower Salt Fork Red
111203	North Fork Red
11120301	Upper North Fork Red
11120302	Middle North Fork Red
11120303	Lower North Fork Red
11120304	Elm Fork Red
1113	Red - Washita
111301	Red-Pease
11130101	Groesbeck-Sandy
11130102	Blue-China
11130103	North Pease
11130104	Middle Pease
11130105	Pease
111302	Red-Lake Texoma
11130201	Farmers-Mud
11130202	Cache
11130203	West Cache
11130204	North Wichita
11130205	South Wichita
11130206	Wichita
11130207	Southern Beaver
11130208	Northern Beaver
11130209	Little Wichita
11130210	Lake Texoma
111303	Washita
11130301	Washita headwaters
11130302	Upper Washita
11130303	Middle Washita
11130304	Lower Washita
1114	Red-Sulphur
111401	Red-Little
11140101	Bois D'arc-Island
11140102	Blue
11140103	Muddy Boggy
11140104	Clear Boggy
11140105	Kiamichi
11140106	Pecan-Waterhole
11140107	Upper Little
11140108	Mountain Fork
11140109	Lower Little
111402	Red-Saline
11140201	Mckinney-Posten Bayous
11140202	Middle Red-Coushatta
11140203	Loggy Bayou
11140204	Red Chute
11140205	Bodcau Bayou
11140206	Bayou Pierre
11140207	Lower Red-Lake Iatt
11140208	Saline Bayou
11140209	Black Lake Bayou
111403	Big Cypress - Sulphur
11140301	Sulphur headwaters
11140302	Lower Sulphur
11140303	White Oak Bayou
11140304	Cross Bayou
11140305	Lake O'the Pines
11140306	Caddo Lake
11140307	Little Cypress
12	Texas-Gulf
1201	Sabine
120100	Sabine
12010001	Upper Sabine
12010002	Middle Sabine
12010003	Lake Fork
12010004	Toledo Bend Reservoir
12010005	Lower Sabine
1202	Neches
120200	Neches
12020001	Upper Neches
12020002	Middle Neches
12020003	Lower Neches
12020004	Upper Angelina
12020005	Lower Angelina
12020006	Village
12020007	Pine Island Bayou
1203	Trinity
120301	Upper Trinity
12030101	Upper West Fork Trinity
12030102	Lower West Fork Trinity
12030103	Elm Fork Trinity
12030104	Denton
12030105	Upper Trinity
12030106	East Fork Trinity
12030107	Cedar
12030108	Richland
12030109	Chambers
120302	Lower Trinity
12030201	Lower Trinity-Tehuacana
12030202	Lower Trinity-Kickapoo
12030203	Lower Trinity
1204	Galveston Bay-San Jacinto
120401	San Jacinto
12040101	West Fork San Jacinto
12040102	Spring
12040103	East Fork San Jacinto
12040104	Buffalo-San Jacinto
120402	Galveston Bay-Sabine Lake
12040201	Sabine Lake
12040202	East Galveston Bay
12040203	North Galveston Bay
12040204	West Galveston Bay
12040205	Austin-Oyster
1205	Brazos headwaters
120500	Brazos headwaters
12050001	Yellow House Draw
12050002	Blackwater Draw
12050003	North Fork Double Mountain Fork
12050004	Double Moutain Fork Brazos
12050005	Running Water Draw
12050006	White
12050007	Salt Fork Brazos
1206	Middle Brazos
120601	Middle Brazos-Clear Fork
12060101	Middle Brazos-Millers
12060102	Upper Clear Fork Brazos
12060103	Paint
12060104	Lower Clear Fork Brazos
12060105	Hubbard
120602	Middle Brazos-Bosque
12060201	Middle Brazos-Palo Pinto
12060202	Middle Brazos-Lake Whitney
12060203	Bosque
12060204	North Bosque
1207	Lower Brazos
120701	Lower Brazos
12070101	Lower Brazos-Little Brazos
12070102	Yegua
12070103	Navasota
12070104	Lower Brazos
120702	Little
12070201	Leon
12070202	Cowhouse
12070203	Lampasas
12070204	Little
12070205	San Gabriel
1208	Upper Colorado
120800	Upper Colorado
12080001	Lost Draw
12080002	Colorado headwaters
12080003	Monument-Seminole Draws
12080004	Mustang Draw
12080005	Johnson Draw
12080006	Sulphur Springs Draw
12080007	Beals
12080008	Upper Colorado
1209	Lower Colorado-San Bernard Coastal
120901	Middle Colorado-Concho
12090101	Middle Colorado-Elm
12090102	South Concho
12090103	Middle Concho
12090104	North Concho
12090105	Concho
12090106	Middle Colorado
12090107	Pecan Bayou
12090108	Jim Ned
12090109	San Saba
12090110	Brady
120902	Middle Colorado-Llano
12090201	Buchanan-Lyndon B
12090202	North Llano
12090203	South Llano
12090204	Llano
12090205	Austin-Travis Lakes
12090206	Pedernales
120903	Lower Colorado
12090301	Lower Colorado-Cummins
12090302	Lower Colorado
120904	San Bernard Coastal
12090401	San Bernard
12090402	East Matagorda Bay
1210	Central Texas Coastal
121001	Lavaca
12100101	Lavaca
12100102	Navidad
121002	Guadalupe
12100201	Upper Guadalupe
12100202	Middle Guadalupe
12100203	San Marcos
12100204	Lower Guadalupe
121003	San Antonio
12100301	Upper San Antonio
12100302	Medina
12100303	Lower San Antonio
12100304	Cibolo
121004	Central Texas Coastal
12100401	Central Matagorda Bay
12100402	West Matagorda Bay
12100403	East San Antonio Bay
12100404	West San Antonio Bay
12100405	Aransas Bay
12100406	Mission
12100407	Aransas
1211	Nueces-Southwestern Texas Coastal
121101	Nueces
12110101	Nueces headwaters
12110102	West Nueces
12110103	Upper Nueces
12110104	Turkey
12110105	Middle Nueces
12110106	Upper Frio
12110107	Hondo
12110108	Lower Frio
12110109	San Miguel
12110110	Atascosa
12110111	Lower Nueces
121102	Southwestern Texas Coastal
12110201	North Corpus Christi Bay
12110202	South Corpus Christi Bay
12110203	North Laguna Madre
12110204	San Fernando
12110205	Baffin Bay
12110206	Palo Blanco
12110207	Central Laguna Madre
12110208	South Laguna Madre
13	Rio Grande
1301	Rio Grande headwaters
130100	Rio Grande headwaters
13010001	Rio Grande headwaters
13010002	Alamosa-Trinchera
13010003	San Luis
13010004	Saguache
13010005	Conejos
1302	Rio Grande-Elephant Butte
130201	Upper Rio Grande
13020101	Upper Rio Grande
13020102	Rio Chama
130202	Rio Grande-Elephant Butte
13020201	Rio Grande-Santa Fe
13020202	Jemez
13020203	Rio Grande-Albuquerque
13020204	Rio Puerco
13020205	Arroyo Chico
13020206	North Plains
13020207	Rio San Jose
13020208	Plains of San Agustin
13020209	Rio Salado
13020210	Jornada Del Muerto
13020211	Elephant Butte Reservoir
1303	Rio Grande-Mimbres
130301	Rio Grande-Caballo
13030101	Caballo
13030102	El Paso-Las Cruces
13030103	Jornada Draw
130302	Mimbres
13030201	Playas Lake
13030202	Mimbres
1304	Rio Grande-Amistad
130401	Rio Grande-Fort Quitman
13040100	Rio Grande-Fort Quitman
130402	Rio Grande-Amistad
13040201	Cibolo-Red Light
13040202	Alamito
13040203	Black Hills-Fresno
13040204	Terlingua
13040205	Big Bend
13040206	Maravillas
13040207	Santiago Draw
13040208	Reagan-Sanderson
13040209	San Francisco
13040210	Lozier Canyon
13040211	Big Canyon
13040212	Amistad Reservoir
130403	Devils
13040301	Upper Devils
13040302	Lower Devils
13040303	Dry Devils
1305	Rio Grande closed basins
130500	Rio Grande closed basins
13050001	Western Estancia
13050002	Eastern Estancia
13050003	Tularosa Valley
13050004	Salt Basin
1306	Upper Pecos
130600	Upper Pecos
13060001	Pecos headwaters
13060002	Pintada Arroyo
13060003	Upper Pecos
13060004	Taiban
13060005	Arroyo Del Macho
13060006	Gallo Arroyo
13060007	Upper Pecos-Long Arroyo
13060008	Rio Hondo
13060009	Rio Felix
13060010	Rio Penasco
13060011	Upper Pecos-Black
1307	Lower Pecos
130700	Lower Pecos
13070001	Lower Pecos-Red Bluff Reservoir
13070002	Delaware
13070003	Toyah
13070004	Salt Draw
13070005	Barrilla Draw
13070006	Coyanosa-Hackberry Draws
13070007	Landreth-Monument Draws
13070008	Lower Pecos
13070009	Tunas
13070010	Independence
13070011	Howard Draw
1308	Rio Grande-Falcon
130800	Rio Grande-Falcon
13080001	Elm-Sycamore
13080002	San Ambrosia-Santa Isabel
13080003	International Falcon Reservoir
1309	Lower Rio Grande
130900	Lower Rio Grande
13090001	Los Olmos
13090002	Lower Rio Grande
14	Upper Colorado
1401	Colorado headwaters
140100	Colorado headwaters
14010001	Colorado headwaters
14010002	Blue
14010003	Eagle
14010004	Roaring Fork
14010005	Colorado headwaters-Plateau
14010006	Parachute-Roan
1402	Gunnison
140200	Gunnison
14020001	East-Taylor
14020002	Upper Gunnison
14020003	Tomichi
14020004	North Fork Gunnison
14020005	Lower Gunnison
14020006	Uncompahange
1403	Upper Colorado-Dolores
140300	Upper Colorado-Dolores
14030001	Westwater Canyon
14030002	Upper Dolores
14030003	San Miguel
14030004	Lower Dolores
14030005	Upper Colorado-Kane Springs
1404	Great Divide - Upper Green
140401	Upper Green
14040101	Upper Green
14040102	New Fork
14040103	Upper Green-Slate
14040104	Big Sandy
14040105	Bitter
14040106	Upper Green-Flaming Gorge Reservoir
14040107	Blacks Fork
14040108	Muddy
14040109	Vermilion
140402	Great Divide closed basin
14040200	Great Divide closed basin
1405	White-Yampa
140500	White - Yampa
14050001	Upper Yampa
14050002	Lower Yampa
14050003	Little Snake
14050004	Muddy
14050005	Upper White
14050006	Piceance-Yellow
14050007	Lower White
1406	Lower Green
140600	Lower Green
14060001	Lower Green-Diamond
14060002	Ashley-Brush
14060003	Duchesne
14060004	Strawberry
14060005	Lower Green-Desolation Canyon
14060006	Willow
14060007	Price
14060008	Lower Green
14060009	San Rafael
1407	Upper Colorado-Dirty Devil
140700	Upper Colorado-Dirty Devil
14070001	Upper Lake Powell
14070002	Muddy
14070003	Fremont
14070004	Dirty Devil
14070005	Escalante
14070006	Lower Lake Powell
14070007	Paria
1408	San Juan
140801	Upper San Juan
14080101	Upper San Juan
14080102	Piedra
14080103	Blanco Canyon
14080104	Animas
14080105	Middle San Juan
14080106	Chaco
14080107	Mancos
140802	Lower San Juan
14080201	Lower San Juan-Four Corners
14080202	Mcelmo
14080203	Montezuma
14080204	Chinle
14080205	Lower San Juan
15	Lower Colorado
1501	Lower Colorado-Lake Mead
150100	Lower Colorado-Lake Mead
15010001	Lower Colorado-Marble Canyon
15010002	Grand Canyon
15010003	Kanab
15010004	Havasu Canyon
15010005	Lake Mead
15010006	Grand Wash
15010007	Hualapai Wash
15010008	Upper Virgin
15010009	Fort Pierce Wash
15010010	Lower Virgin
15010011	White
15010012	Muddy
15010013	Meadow Valley Wash
15010014	Detrital Wash
15010015	Las Vegas Wash
1502	Little Colorado
150200	Little Colorado
15020001	Little Colorado headwaters
15020002	Upper Little Colorado
15020003	Carrizo Wash
15020004	Zuni
15020005	Silver
15020006	Upper Puerco
15020007	Lower Puerco
15020008	Middle Little Colorado
15020009	Leroux Wash
15020010	Chevelon Canyon
15020011	Cottonwood Wash
15020012	Corn-Oraibi
15020013	Polacca Wash
15020014	Jadito Wash
15020015	Canyon Diablo
15020016	Lower Little Colorado
15020017	Dinnebito Wash
15020018	Moenkopi Wash
1503	Lower Colorado
150301	Lower Colorado
15030101	Havasu-Mohave Lakes
15030102	Piute Wash
15030103	Sacramento Wash
15030104	Imperial Reservoir
15030105	Bouse Wash
15030106	Tyson Wash
15030107	Lower Colorado
15030108	Yuma Desert
150302	Bill Williams
15030201	Big Sandy
15030202	Burro
15030203	Santa Maria
15030204	Bill Williams
1504	Upper Gila
150400	Upper Gila
15040001	Upper Gila
15040002	Upper Gila-Mangas
15040003	Animas Valley
15040004	San Francisco
15040005	Upper Gila-San Carlos Reservoir
15040006	San Simon
15040007	San Carlos
1505	Middle Gila
150501	Middle Gila
15050100	Middle Gila
150502	San Pedro-Willcox
15050201	Willcox Playa
15050202	Upper San Pedro
15050203	Lower San Pedro
150503	Santa Cruz
15050301	Upper Santa Cruz
15050302	Rillito
15050303	Lower Santa Cruz
15050304	Brawley Wash
15050305	Aguirre Valley
15050306	Santa Rosa Wash
1506	Salt
150601	Salt
15060101	Black
15060102	White
15060103	Upper Salt
15060104	Carrizo
15060105	Tonto
15060106	Lower Salt
150602	Verde
15060201	Big Chino-Williamson Valley
15060202	Upper Verde
15060203	Lower Verde
1507	Lower Gila
150701	Lower Gila-Agua Fria
15070101	Lower Gila-Painted Rock Reservoir
15070102	Agua Fria
15070103	Hassayampa
15070104	Centennial Wash
150702	Lower Gila
15070201	Lower Gila
15070202	Tenmile Wash
15070203	San Cristobal Wash
1508	Sonora
150801	Rio Sonoyta
15080101	San Simon Wash
15080102	Rio Sonoyta
15080103	Tule Desert
150802	Rio De La Concepcion
15080200	Rio De La Concepcion
150803	Rio De Bavispe
15080301	Whitewater Draw
15080302	San Bernardino Valley
15080303	Cloverdale
16	Great Basin
1601	Bear
160101	Upper Bear
16010101	Upper Bear
16010102	Central Bear
160102	Lower Bear
16010201	Bear Lake
16010202	Middle Bear
16010203	Little Bear-Logan
16010204	Lower Bear-Malad
1602	Great Salt Lake
160201	Weber
16020101	Upper Weber
16020102	Lower Weber
160202	Jordan
16020201	Utah Lake
16020202	Spanish Fork
16020203	Provo
16020204	Jordan
160203	Great Salt Lake
16020301	Hamlin-Snake Valleys
16020302	Pine Valley
16020303	Tule Valley
16020304	Rush-Tooele Valleys
16020305	Skull Valley
16020306	Southern Great Salt Lake Desert
16020307	Pilot-Thousand Springs
16020308	Northern Great Salt Lake Desert
16020309	Curlew Valley
16020310	Great Salt Lake
1603	Escalante Desert-Sevier Lake
160300	Escalante Desert-Sevier Lake
16030001	Upper Sevier
16030002	East Fork Sevier
16030003	Middle Sevier
16030004	San Pitch
16030005	Lower Sevier
16030006	Escalante Desert
16030007	Beaver Bottoms-Upper Beaver
16030008	Lower Beaver
16030009	Sevier Lake
1604	Black Rock Desert-Humboldt
160401	Humboldt
16040101	Upper Humboldt
16040102	North Fork Humboldt
16040103	South Fork Humboldt
16040104	Pine
16040105	Middle Humboldt
16040106	Rock
16040107	Reese
16040108	Lower Humboldt
16040109	Little Humboldt
160402	Black Rock Desert
16040201	Upper Quinn
16040202	Lower Quinn
16040203	Smoke Creek Desert
16040204	Massacre Lake
16040205	Thousand-Virgin
1605	Central Lahontan
160501	Truckee
16050101	Lake Tahoe
16050102	Truckee
16050103	Pyramid-Winnemucca Lakes
16050104	Granite Springs Valley
160502	Carson
16050201	Upper Carson
16050202	Middle Carson
16050203	Carson Desert
160503	Walker
16050301	East Walker
16050302	West Walker
16050303	Walker
16050304	Walker Lake
1606	Central Nevada Desert Basins
160600	Central Nevada Desert Basins
16060001	Dixie Valley
16060002	Gabbs Valley
16060003	Southern Big Smoky Valley
16060004	Northern Big Smoky Valley
16060005	Diamond-Monitor Valleys
16060006	Little Smoky-Newark Valleys
16060007	Long-Ruby Valleys
16060008	Spring-Steptoe Valleys
16060009	Dry Lake Valley
16060010	Fish Lake-Soda Spring Valleys
16060011	Ralston-Stone Cabin Valleys
16060012	Hot Creek-Railroad Valleys
16060013	Cactus-Sarcobatus Flats
16060014	Sand Spring-Tikaboo Valleys
16060015	Ivanpah-Pahrump Valleys
17	Pacific Northwest
1701	Kootenai-Pend Oreille-Spokane
170101	Kootenai
17010101	Upper Kootenai
17010102	Fisher
17010103	Yaak
17010104	Lower Kootenai
17010105	Moyie
170102	Pend Oreille
17010201	Upper Clark Fork
17010202	Flint-Rock
17010203	Blackfoot
17010204	Middle Clark Fork
17010205	Bitterroot
17010206	North Fork Flathead
17010207	Middle Fork Flathead
17010208	Flathead Lake
17010209	South Fork Flathead
17010210	Stillwater
17010211	Swan
17010212	Lower Flathead
17010213	Lower Clark Fork
17010214	Pend Oreille Lake
17010215	Priest
17010216	Pend Oreille
170103	Spokane
17010301	Upper Coeur D'alene
17010302	South Fork Coeur D'alene
17010303	Coeur D'alene Lake
17010304	St. Joe
17010305	Upper Spokane
17010306	Hangman
17010307	Lower Spokane
17010308	Little Spokane
1702	Upper Columbia
170200	Upper Columbia
17020001	Franklin D. Roosevelt Lake
17020002	Kettle
17020003	Colville
17020004	Sanpoil
17020005	Chief Joseph
17020006	Okanogan
17020007	Similkameen
17020008	Methow
17020009	Lake Chelan
17020010	Upper Columbia-Entiat
17020011	Wenatchee
17020012	Moses Coulee
17020013	Upper Crab
17020014	Banks Lake
17020015	Lower Crab
17020016	Upper Columbia-Priest Rapids
1703	Yakima
170300	Yakima
17030001	Upper Yakima
17030002	Naches
17030003	Lower Yakima, Washington
1704	Upper Snake
170401	Snake headwaters
17040101	Snake headwaters
17040102	Gros Ventre
17040103	Greys-Hobock
17040104	Palisades
17040105	Salt
170402	Upper Snake
17040201	Idaho Falls
17040202	Upper Henrys
17040203	Lower Henrys
17040204	Teton
17040205	Willow
17040206	American Falls
17040207	Blackfoot
17040208	Portneuf
17040209	Lake Walcott
17040210	Raft
17040211	Goose
17040212	Upper Snake-Rock
17040213	Salmon Falls
17040214	Beaver-Camas
17040215	Medicine Lodge
17040216	Birch
17040217	Little Lost
17040218	Big Lost
17040219	Big Wood
17040220	Camas
17040221	Little Wood
1705	Middle Snake
170501	Middle Snake-Boise
17050101	C. J. Idaho
17050102	Bruneau
17050103	Middle Snake-Succor
17050104	Upper Owyhee
17050105	South Fork Owyhee
17050106	East Little Owyhee. Nevada,
17050107	Middle Owyhee
17050108	Jordan
17050109	Crooked-Rattlesnake
17050110	Lower Owyhee
17050111	North and Middle Forks Boise
17050112	Boise-Mores
17050113	South Fork Boise
17050114	Lower Boise
17050115	Middle Snake-Payette
17050116	Upper Malheur
17050117	Lower Malheur
17050118	Bully
17050119	Willow
17050120	South Fork Payette
17050121	Middle Fork Payette
17050122	Payette
17050123	North Fork Payette
17050124	Weiser
170502	Middle Snake-Powder
17050201	Brownlee Reservoir
17050202	Burnt
17050203	Powder
1706	Lower Snake
170601	Lower Snake
17060101	Hells Canyon
17060102	Imnaha
17060103	Lower Snake-Asotin
17060104	Upper Grande Ronde
17060105	Wallowa
17060106	Lower Grande Ronde
17060107	Lower Snake-Tucannon
17060108	Palouse
17060109	Rock
17060110	Lower Snake
170602	Salmon
17060201	Upper Salmon
17060202	Pahsimeroi
17060203	Middle Salmon-Panther
17060204	Lemhi
17060205	Upper Middle Fork Salmon
17060206	Lower Middle Fork Salmon
17060207	Middle Salmon-Chamberlain
17060208	South Fork Salmon
17060209	Lower Salmon
17060210	Little Salmon
170603	Clearwater
17060301	Upper Selway
17060302	Lower Selway
17060303	Lochsa
17060304	Middle Fork Clearwater
17060305	South Fork Clearwater
17060306	Clearwater
17060307	Upper North Fork Clearwater
17060308	Lower North Fork Clearwater
1707	Middle Columbia
170701	Middle Columbia
17070101	Middle Columbia-Lake Wallula
17070102	Walla Walla
17070103	Umatilla
17070104	Willow
17070105	Middle Columbia-Hood
17070106	Klickitat
170702	John Day
17070201	Upper John Day
17070202	North Fork John Day
17070203	Middle Fork John Day
17070204	Lower John Day
170703	Deschutes
17070301	Upper Deschutes
17070302	Little Deschutes
17070303	Beaver-South Fork
17070304	Upper Crooked
17070305	Lower Crooked
17070306	Lower Deschutes
17070307	Trout
1708	Lower Columbia
170800	Lower Columbia
17080001	Lower Columbia-Sandy
17080002	Lewis
17080003	Lower Columbia-Clatskanie
17080004	Upper Cowlitz
17080005	Lower Cowlitz
17080006	Lower Columbia
1709	Willamette
170900	Willamette
17090001	Middle Fork Willamette
17090002	Coast Fork Willamette
17090003	Upper Willamette
17090004	Mckenzie
17090005	North Santiam
17090006	South Santiam
17090007	Middle Willamette
17090008	Yamhill
17090009	Molalla-Pudding
17090010	Tualatin
17090011	Clackamas
17090012	Lower Willamette
1710	Oregon-Washington Coastal
171001	Washington Coastal
17100101	Hoh-Quillayute
17100102	Queets-Quinault
17100103	Upper Chehalis
17100104	Lower Chehalis
17100105	Grays Harbor
17100106	Willapa Bay
171002	Northern Oregon Coastal
17100201	Necanicum
17100202	Nehalem
17100203	Wilson-Trask-Nestucca
17100204	Siletz-Yaquina
17100205	Alsea
17100206	Siuslaw
17100207	Siltcoos
171003	Southern Oregon Coastal
17100301	North Umpqua
17100302	South Umpqua
17100303	Umpqua
17100304	Coos
17100305	Coquille
17100306	Sixes
17100307	Upper Rogue
17100308	Middle Rogue
17100309	Applegate
17100310	Lower Rogue
17100311	Illinois
17100312	Chetco
1711	Puget Sound
171100	Puget Sound
17110001	Fraser
17110002	Strait of Georgia
17110003	San Juan Islands
17110004	Nooksack
17110005	Upper Skagit
17110006	Sauk
17110007	Lower Skagit
17110008	Stillaguamish
17110009	Skykomish
17110010	Snoqualmie
17110011	Snohomish
17110012	Lake Washington
17110013	Duwamish
17110014	Puyallup
17110015	Nisqually
17110016	Deschutes
17110017	Skokomish
17110018	Hood Canal
17110019	Puget Sound
17110020	Dungeness-Elwha
17110021	Crescent-Hoko
1712	Oregon closed basins
171200	Oregon closed basins
17120001	Harney-Malheur Lakes
17120002	Silvies
17120003	Donner Und Blitzen
17120004	Silver
17120005	Summer Lake
17120006	Lake Abert
17120007	Warner Lakes
17120008	Guano
17120009	Alvord Lake
18	California
1801	Klamath-Northern California Coastal
180101	Northern California Coastal
18010101	Smith
18010102	Mad-Redwood
18010103	Upper Eel
18010104	Middle Fork Eel
18010105	Lower Eel
18010106	South Fork Eel
18010107	Mattole
18010108	Big-Navarro-Garcia
18010109	Gualala-Salmon
18010110	Russian
18010111	Bodega Bay
180102	Klamath
18010201	Williamson
18010202	Sprague
18010203	Upper Klamath Lake
18010204	Lost
18010205	Butte
18010206	Upper Klamath
18010207	Shasta
18010208	Scott
18010209	Lower Klamath
18010210	Salmon
18010211	Trinity
18010212	South Fork Trinity
1802	Sacramento
180200	Upper Sacramento
18020001	Goose Lake
18020002	Upper Pit
18020003	Lower Pit
18020004	Mccloud
18020005	Sacramento headwaters
180201	Lower Sacramento
18020101	Sacramento-Lower Cow-Lower Clear
18020102	Lower Cottonwood
18020103	Sacramento-Lower Thomes
18020104	Sacramento-Stone Corral
18020105	Lower Butte
18020106	Lower Feather
18020107	Lower Yuba
18020108	Lower Bear
18020109	Lower Sacramento
18020110	Lower Cache
18020111	Lower American
18020112	Sacramento-Upper Clear
18020113	Cottonwood headwaters
18020114	Upper Elder-Upper Thomes
18020115	Upper Stony
18020116	Upper Cache
18020117	Upper Putah
18020118	Upper Cow-Battle
18020119	Mill-Big Chico
18020120	Upper Butte
18020121	North Fork Feather
18020122	East Branch North Fork Feather
18020123	Middle Fork Feather
18020124	Honcut headwaters
18020125	Upper Yuba
18020126	Upper Bear
18020127	Upper Coon-Upper Auburn
18020128	North Fork American
18020129	South Fork American
1803	Tulare-Buena Vista Lakes
180300	Tulare-Buena Vista Lakes
18030001	Upper Kern
18030002	South Fork Kern
18030003	Middle Kern-Upper Tehachapi-
18030004	Upper Poso
18030005	Upper Deer-Upper White
18030006	Upper Tule
18030007	Upper Kaweah
18030008	Mill
18030009	Upper Dry
18030010	Upper King
18030011	Upper Los Gatos-Avenal
18030012	Tulare-Buena Vista Lakes
1804	San Joaquin
180400	San Joaquin
18040001	Middle San Joaquin-Lower
18040002	Middle San Joaquin-Lower
18040003	San Joaquin Delta
18040004	Lower Calaveras-Mormon Slough
18040005	Lower Cosumnes-Lower Mokelumne
18040006	Upper San Joaquin
18040007	Upper Chowchilla-Upper Fresno
18040008	Upper Merced
18040009	Upper Tuolumne
18040010	Upper Stanislaus
18040011	Upper Calaveras
18040012	Upper Mokelumne
18040013	Upper Cosumnes
18040014	Panoche-San Luis Reservoir
1805	San Francisco Bay
180500	San Francisco Bay
18050001	Suisun Bay
18050002	San Pablo Bay
18050003	Coyote
18050004	San Francisco Bay
18050005	Tomales-Drake Bays
18050006	San Francisco Coastal South
1806	Central California Coastal
180600	Central California Coastal
18060001	San Lorenzo-Soquel
18060002	Pajaro
18060003	Carrizo Plain
18060004	Estrella
18060005	Salinas
18060006	Central Coastal
18060007	Cuyama
18060008	Santa Maria
18060009	San Antonio
18060010	Santa Ynez
18060011	Alisal-Elkhorn Sloughs
18060012	Carmel
18060013	Santa Barbara Coastal
18060014	Santa Barbara Channel Islands
1807	Southern California Coastal
180701	Ventura-San Gabriel Coastal
18070101	Ventura
18070102	Santa Clara
18070103	Calleguas
18070104	Santa Monica Bay
18070105	Los Angeles
18070106	San Gabriel
18070107	San Pedro Channel Islands
180702	Santa Ana
18070201	Seal Beach
18070202	San Jacinto
18070203	Santa Ana
18070204	Newport Bay
180703	Laguna-San Diego Coastal
18070301	Aliso-San Onofre
18070302	Santa Margarita
18070303	San Luis Rey-Escondido
18070304	San Diego
18070305	Cottonwood-Tijuana
1808	North Lahontan
180800	North Lahontan
18080001	Surprise Valley
18080002	Madeline Plains
18080003	Honey-Eagle Lakes
1809	Northern Mojave-Mono Lake
180901	Mono-Owens Lakes
18090101	Mono Lake
18090102	Crowley Lake
18090103	Owens Lake
180902	Northern Mojave
18090201	Eureka-Saline Valleys
18090202	Upper Amargosa
18090203	Death Valley-Lower Amargosa
18090204	Panamint Valley
18090205	Indian Wells-Searles Valleys
18090206	Antelope-Fremont Valleys
18090207	Coyote-Cuddeback Lakes
18090208	Mojave
1810	Southern Mojave-Salton Sea
181001	Southern Mojave
18100100	Southern Mojave
181002	Salton Sea
18100200	Salton Sea
19	Alaska
1901	Southeast
190101	Southern Southeast
19010101	Southeast Mainland
19010102	Ketchikan
19010103	Prince of Wales
190102	Central Southeast
19010201	Mainland
19010202	Kuiu-Kupreanof-Mitkof-Etolin-Zarembo-Wrangell Islands
19010203	Baranof-Chichagof Islands
19010204	Admiralty Island
190103	Northern Southeast
19010301	Lynn Canal
19010302	Glacier Bay
19010303	Chilkat-Skagway Rivers
190104	Gulf of Alaska
19010401	Yakutat Bay
19010402	Bering Glacier
1902	Southcentral
190201	Copper River
19020101	Upper Copper River
19020102	Middle Copper River
19020103	Chitina River
19020104	Lower Copper River
190202	Prince William Sound
19020201	Eastern Prince William Sound
19020202	Western Prince William Sound
190203	Kenai Peninsula
19020301	Lower Kenai Peninsula
19020302	Upper Kenai Peninsula
190204	Knik Arm
19020401	Anchorage
19020402	Matansuka
190205	Susitna River
19020501	Upper Susitna River
19020502	Chulitna River
19020503	Talkeetna River
19020504	Yentna River
19020505	Lower Susitna River
190206	Western Cook Inlet
19020601	Redoubt-Trading Bays
19020602	Tuxdeni-Kamishak Bays
190207	Kodiak-Shelikof
19020701	Kodiak-Afognak Islands
19020702	Shelikof Straight
1903	Southwest
190301	Aleutian Islands
19030101	Cold Bay
19030102	Fox Islands
19030103	Western Aleutian
19030104	Pribilof Islands
190302	Kvichak-Port Heiden
19030201	Port Heiden
19030202	Ugashik Bay
19030203	Egegik Bay
19030204	Naknek
19030205	Lake Clark
19030206	Lake Iliamna
190303	Nushagak River
19030301	Upper Nushagak River
19030302	Mulchatna River
19030303	Lower Nushagak River
19030304	Wood River
19030305	Togiak
190304	Upper Kuskokwim River
19030401	North Fork Kuskokwim River
19030402	Farewll Lake
19030403	Takotna River
19030404	Holitna River
19030405	Stony River
190305	Lower Kuskokwim River
19030501	Aniak
19030502	Kuskokwim Delta
19030503	Nunavak-St. Matthew Islands
1904	Yukon
190401	Canada
19040101	White River
19040102	Ladue River
19040103	Sixtymile River
19040104	Fortymile River
190402	Porcupine River
19040201	Old Crow River
19040202	Coleen River
19040203	Sheenjek River
19040204	Black River
19040205	Porcupine Flats
190403	Chandalar-Christian Rivers
19040301	Middle Fork-North Fork Chandalar Rivers
19040302	East Fork Chandalar River
19040303	Christian River
19040304	Lower Chandalar River
190404	Upper Yukon River
19040401	Eagle To Circle
19040402	Birch-Beaver Creeks
19040403	Yukon Flats
19040404	Ramparts
190405	Tanana River
19040501	Nebesna-Chisana Rivers
19040502	Tok
19040503	Healy Lake
19040504	Delta River
19040505	Salcha River
19040506	Chena River
19040507	Tanana River
19040508	Nenana River
19040509	Tolovana River
19040510	Kantishna River
19040511	Lower Tanana River
190406	Koyukuk River
19040601	Upper Koyukuk River
19040602	South Fork Koyukuk River
19040603	Alatna River
19040604	Kanuti River
19040605	Allakaket River
19040606	Huslia River
19040607	Dulbi River
19040608	Koyukuk Flats
19040609	Kateel River
190407	Central Yukon
19040701	Tozitna River
19040702	Nowitna River
19040703	Melozitna River
19040704	Ramparts to Ruby
19040705	Galena
190408	Lower Yukon
19040801	Anvik River
19040802	Upper Innoko River
19040803	Lower Innoko River
19040804	Anvik to Pilot Station
19040805	Yukon Delta
1905	Northwest
190501	Norton Sound
19050101	St. Lawrence Island
19050102	Unalakleet
19050103	Norton Bay
19050104	Nome
19050105	Imuruk Basin
190502	Northern Seward Peninsula
19050201	Shishmaref
19050202	Goodhope-Spafarief Bay
19050203	Buckland River
190503	Kobuk-Selawik Rivers
19050301	Selawik Lake
19050302	Upper Kobuk River
19050303	Middle Kobuk River
19050304	Lower Kobuk River
190504	Noatak River-Lisburne Peninsula
19050401	Upper Noatak River
19050402	Middle Noatak River
19050403	Lower Noatak River
19050404	Wulik-Kivalina Rivers
19050405	Lisburne Peninsula
1906	Arctic
190601	Western Arctic
19060101	Kukpowruk River
19060102	Kokolik River
19060103	Utukok River
190602	Barrow
19060201	Kuk River
19060202	Northwest Coast
19060203	Meade River
19060204	Ikpikpuk River
19060205	Harrison Bay
190603	Colville River
19060301	Upper Colville River
19060302	Killik River
19060303	Chandler-Anaktuvuk Rivers
19060304	Lower Colville River
190604	Prudhoe Bay
19060401	Kuparuk River
19060402	Sagavanirktok River
19060403	Mikkelson Bay
190605	Eastern Arctic
19060501	Canning River
19060502	Camden Bay
19060503	Beaufort Lagoon
20	Hawaii
2001	Hawaii
200100	Hawaii
20010000	Hawaii
2002	Maui
200200	Maui
20020000	Maui
2003	Kahoolawe
200300	Kahoolawe
20030000	Kahoolawe
2004	Lanai
200400	Lanai
20040000	Lanai
2005	Molokai
200500	Molokai
20050000	Molokai
2006	Oahu
200600	Oahu
20060000	Oahu
2007	Kauai
200700	Kauai
20070000	Kauai
2008	Niihau
200800	Niihau
20080000	Niihau
2009	Northwestern Hawaiian Islands
200900	Northwestern Hawaiian Islands
20090000	Northwestern Hawaiian Islands
21	Caribbean
2101	Puerto Rico
210100	Puerto Rico
21010001	Interior Puerto Rico
21010002	Cibuco-Guajataca
21010003	Culebrinas-Guanajibo
21010004	Southern Puerto Rico
21010005	Eastern Puerto Rico
21010006	Puerto Rican Islands
2102	Virgin Islands
210200	Virgin Islands
21020001	St. John-St. Thomas
21020002	St. Croix
2103	Caribbean Outlying Areas
210300	Caribbean Outlying Areas
21030001	Canal Zone
21030002	Navassa
21030003	Roncador-Serrana
</pre>
         </td>
        </tr>
      </tbody>
    </table>
    <div align="center" class="bottombox"><span class="info"></span></div>
    <!--#include virtual="/inc/footer_water.html" -->
  <!--#include virtual="/inc/footer.html" -->
</body>
pip</html>"""



"""

.. currentmodule:: basin3d.monitor

:synopsis: The BASIN-3D monitoring module
:module author: Val Hendrix <vhendrix@lbl.gov>

This module holds the logging functionality of BASIN-3D. It supports
logging the messages and their contexts

.. contents:: Contents
    :local:
    :backlinks: top

Functions
----------------
* :func:`get_logger` - Get the Basin3DLogger.
* :func:`configure` - Configure logging in BASIN-3D



"""
from basin3d.core.monitor import configure, get_logger

__all__ = ['configure', 'get_logger']


from basin3d import monitor, synthesis
from importlib.metadata import version, PackageNotFoundError

try:
    __version__ = version("basin3d")
except PackageNotFoundError:
    # package is not installed
    pass
__all__ = ['monitor', 'synthesis']